@@ -26,7 +26,6 @@
 #include <urcu.h>
 #include "knot/conf/conf.h"
 #include "knot/common.h"
-#include "knot/other/error.h"
 
 /*
  * Defaults.
@@ -70,75 +69,31 @@ void cf_error(void *scanner, const char *msg)
 			 new_config->filename, msg, lineno, text);
 
 
-	_parser_res = KNOTD_EPARSEFAIL;
+	_parser_res = KNOT_EPARSEFAIL;
 }
 
-/*
- * Config helper functions.
- */
-
-/*! \brief Free TSIG key. */
-static void key_free(conf_key_t *k)
+static int conf_ztree_compare(void *p1, void *p2)
 {
-	/* Secure erase. */
-	if (k->k.secret) {
-		memset(k->k.secret, 0, strlen(k->k.secret));
-	}
-	free(k->k.secret);
-	knot_dname_free(&k->k.name);
-	free(k);
+	return knot_dname_compare((knot_dname_t*)p1, (knot_dname_t*)p2);
 }
 
-/*! \brief Free config interfaces. */
-static void iface_free(conf_iface_t *iface)
+static void conf_ztree_free(void *node, void *data)
 {
-	if (!iface) {
-		return;
-	}
-
-	free(iface->name);
-	free(iface->address);
-	free(iface);
+	UNUSED(data);
+	knot_dname_t *zname = (knot_dname_t*)node;
+	knot_dname_free(&zname);
 }
 
-/*! \brief Free config logs. */
-static void log_free(conf_log_t *log)
+static void conf_parse_begin(conf_t *conf)
 {
-	if (!log) {
-		return;
-	}
-
-	if (log->file) {
-		free(log->file);
-	}
-
-	/* Free loglevel mapping. */
-	node *n = 0, *nxt = 0;
-	WALK_LIST_DELSAFE(n, nxt, log->map) {
-		free((conf_log_map_t*)n);
-	}
-
-	free(log);
+	conf->zone_tree = gen_tree_new(conf_ztree_compare);
 }
 
-/*! \brief Free config zones. */
-static void zone_free(conf_zone_t *zone)
+static void conf_parse_end(conf_t *conf) 
 {
-	if (!zone) {
-		return;
+	if (conf->zone_tree) {
+		gen_tree_destroy(&conf->zone_tree, conf_ztree_free, NULL);
 	}
-
-	/* Free ACL lists. */
-	WALK_LIST_FREE(zone->acl.xfr_in);
-	WALK_LIST_FREE(zone->acl.xfr_out);
-	WALK_LIST_FREE(zone->acl.notify_in);
-	WALK_LIST_FREE(zone->acl.notify_out);
-
-	free(zone->name);
-	free(zone->file);
-	free(zone->db);
-	free(zone->ixfr_db);
-	free(zone);
 }
 
 /*!
@@ -175,7 +130,7 @@ static int conf_process(conf_t *conf)
 	if (conf->storage == NULL) {
 		conf->storage = strdup("/var/lib/"PROJECT_EXEC);
 		if (conf->storage == NULL) {
-			return KNOTD_ENOMEM;
+			return KNOT_ENOMEM;
 		}
 	}
 	
@@ -187,25 +142,25 @@ static int conf_process(conf_t *conf)
 	if (stat(conf->storage, &st) == -1) {
 		log_server_error("Could not open storage directory '%s'\n", conf->storage);
 		// I assume that conf->* is freed elsewhere
-		return KNOTD_EINVAL;
+		return KNOT_EINVAL;
 	}
 
 	// Storage directory is a directory?
 	if (S_ISDIR(st.st_mode) == 0) {
 		log_server_error("Configured storage '%s' not a directory\n", conf->storage);
-		return KNOTD_EINVAL;
+		return KNOT_EINVAL;
 	}
 
 	// Create PID file
 	if (conf->pidfile == NULL) {
 		conf->pidfile = strcdup(conf->storage, "/" PID_FILE);
 		if (conf->pidfile == NULL) {
-			return KNOTD_ENOMEM;
+			return KNOT_ENOMEM;
 		}
 	}
 
 	// Postprocess zones
-	int ret = KNOTD_EOK;
+	int ret = KNOT_EOK;
 	node *n = 0;
 	WALK_LIST (n, conf->zones) {
 		conf_zone_t *zone = (conf_zone_t*)n;
@@ -214,6 +169,11 @@ static int conf_process(conf_t *conf)
 		if (zone->dbsync_timeout < 0) {
 			zone->dbsync_timeout = conf->dbsync_timeout;
 		}
+		
+		// Default policy for ixfr-from-differences
+		if (zone->build_diffs < 0) {
+			zone->build_diffs = conf->build_diffs;
+		}
 
 		// Default policy for semantic checks
 		if (zone->enable_checks < 0) {
@@ -240,6 +200,11 @@ static int conf_process(conf_t *conf)
 			zone->ixfr_fslimit = conf->ixfr_fslimit;
 		}
 		
+		// Default zone file
+		if (zone->file == NULL) {
+			zone->file = strcdup(zone->name, ".zone");
+		}
+		
 		// Relative zone filenames should be relative to storage
 		if (zone->file[0] != '/') {
 			size_t prefix_len = strlen(conf->storage) + 1; // + '\0'
@@ -252,7 +217,7 @@ static int conf_process(conf_t *conf)
 				free(zone->file);
 				zone->file = ap;
 			} else {
-				ret = KNOTD_ENOMEM;
+				ret = KNOT_ENOMEM;
 				continue;
 			}
 		}
@@ -261,7 +226,7 @@ static int conf_process(conf_t *conf)
 		zone->file = strcpath(zone->file);
 		if (zone->file == NULL) {
 			zone->db = NULL;
-			ret = KNOTD_ENOMEM;
+			ret = KNOT_ENOMEM;
 			continue;
 		}
 
@@ -272,7 +237,7 @@ static int conf_process(conf_t *conf)
 		char *dest = malloc(size);
 		if (dest == NULL) {
 			zone->db = NULL; /* Not enough memory. */
-			ret = KNOTD_ENOMEM; /* Error report. */
+			ret = KNOT_ENOMEM; /* Error report. */
 			continue;
 		}
 		char *dpos = dest;
@@ -286,7 +251,12 @@ static int conf_process(conf_t *conf)
 			*dpos = '\0';
 		}
 
+		/* Copy origin and remove bad characters. */
 		memcpy(dpos, zone->name, zname_len + 1);
+		for (int i = 0; i < zname_len; ++i) {
+			if (dpos[i] == '/') dpos[i] = '_';
+		}
+		
 		memcpy(dpos + zname_len, "db", 3);
 		zone->db = dest;
 
@@ -296,19 +266,29 @@ static int conf_process(conf_t *conf)
 		dest = malloc(size);
 		if (dest == NULL) {
 			zone->ixfr_db = NULL; /* Not enough memory. */
-			ret = KNOTD_ENOMEM; /* Error report. */
+			ret = KNOT_ENOMEM; /* Error report. */
 			continue;
 		}
-		strncpy(dest, conf->storage, stor_len + 1);
+		dpos = dest;
+		memcpy(dpos, conf->storage, stor_len + 1);
+		dpos += stor_len;
 		if (conf->storage[stor_len - 1] != '/') {
-			strncat(dest, "/", 1);
+			*(dpos++) = '/';	
+			*dpos = '\0';
 		}
 
 		const char *dbext = "diff.db";
-		strncat(dest, zone->name, zname_len);
-		strncat(dest, dbext, strlen(dbext));
+		memcpy(dpos, zone->name, zname_len + 1);
+		for (int i = 0; i < zname_len; ++i) {
+			if (dpos[i] == '/') dpos[i] = '_';
+		}
+		memcpy(dpos + zname_len, dbext, strlen(dbext) + 1);
 		zone->ixfr_db = dest;
 	}
+	
+	/* Update UID and GID. */
+	if (conf->uid < 0) conf->uid = getuid();
+	if (conf->gid < 0) conf->gid = getgid();
 
 	return ret;
 }
@@ -385,22 +365,23 @@ void __attribute__ ((destructor)) conf_deinit()
 static int conf_fparser(conf_t *conf)
 {
 	if (!conf->filename) {
-		return KNOTD_EINVAL;
+		return KNOT_EINVAL;
 	}
 
-	int ret = KNOTD_EOK;
+	int ret = KNOT_EOK;
 	pthread_mutex_lock(&_parser_lock);
+	
 	// {
 	// Hook new configuration
 	new_config = conf;
 	FILE *f = fopen(conf->filename, "r");
 	if (f == 0) {
 		pthread_mutex_unlock(&_parser_lock);
-		return KNOTD_ENOENT;
+		return KNOT_ENOENT;
 	}
 
 	// Parse config
-	_parser_res = KNOTD_EOK;
+	_parser_res = KNOT_EOK;
 	new_config->filename = conf->filename;
 	void *sc = NULL;
 	cf_lex_init(&sc);
@@ -411,6 +392,7 @@ static int conf_fparser(conf_t *conf)
 	fclose(f);
 	// }
 	pthread_mutex_unlock(&_parser_lock);
+	
 	return ret;
 }
 
@@ -420,17 +402,17 @@ static int conf_fparser(conf_t *conf)
 static int conf_strparser(conf_t *conf, const char *src)
 {
 	if (!src) {
-		return KNOTD_EINVAL;
+		return KNOT_EINVAL;
 	}
 
-	int ret = KNOTD_EOK;
+	int ret = KNOT_EOK;
 	pthread_mutex_lock(&_parser_lock);
 	// {
 	// Hook new configuration
 	new_config = conf;
 
 	// Parse config
-	_parser_res = KNOTD_EOK;
+	_parser_res = KNOT_EOK;
 	char *oldfn = new_config->filename;
 	new_config->filename = "(stdin)";
 	void *sc = NULL;
@@ -475,6 +457,7 @@ conf_t *conf_new(const char* path)
 	c->ixfr_fslimit = -1;
 	c->uid = -1;
 	c->gid = -1;
+	c->build_diffs = 0; /* Disable by default. */
 
 	return c;
 }
@@ -484,7 +467,7 @@ int conf_add_hook(conf_t * conf, int sections,
 {
 	conf_hook_t *hook = malloc(sizeof(conf_hook_t));
 	if (!hook) {
-		return KNOTD_ENOMEM;
+		return KNOT_ENOMEM;
 	}
 
 	hook->sections = sections;
@@ -493,14 +476,16 @@ int conf_add_hook(conf_t * conf, int sections,
 	add_tail(&conf->hooks, &hook->n);
 	++conf->hooks_count;
 
-	return KNOTD_EOK;
+	return KNOT_EOK;
 }
 
 int conf_parse(conf_t *conf)
 {
 	/* Parse file. */
+	conf_parse_begin(conf);
 	int ret = conf_fparser(conf);
-
+	conf_parse_end(conf);
+	
 	/* Postprocess config. */
 	if (ret == 0) {
 		ret = conf_process(conf);
@@ -509,16 +494,18 @@ int conf_parse(conf_t *conf)
 	}
 
 	if (ret < 0) {
-		return KNOTD_EPARSEFAIL;
+		return KNOT_EPARSEFAIL;
 	}
 
-	return KNOTD_EOK;
+	return KNOT_EOK;
 }
 
 int conf_parse_str(conf_t *conf, const char* src)
 {
 	/* Parse config from string. */
+	conf_parse_begin(conf);
 	int ret = conf_strparser(conf, src);
+	conf_parse_end(conf);
 
 	/* Postprocess config. */
 	conf_process(conf);
@@ -527,10 +514,10 @@ int conf_parse_str(conf_t *conf, const char* src)
 	conf_update_hooks(conf);
 
 	if (ret < 0) {
-		return KNOTD_EPARSEFAIL;
+		return KNOT_EPARSEFAIL;
 	}
 
-	return KNOTD_EOK;
+	return KNOT_EOK;
 }
 
 void conf_truncate(conf_t *conf, int unload_hooks)
@@ -553,33 +540,33 @@ void conf_truncate(conf_t *conf, int unload_hooks)
 
 	// Free keys
 	WALK_LIST_DELSAFE(n, nxt, conf->keys) {
-		key_free((conf_key_t *)n);
+		conf_free_key((conf_key_t *)n);
 	}
 
 	// Free interfaces
 	WALK_LIST_DELSAFE(n, nxt, conf->ifaces) {
-		iface_free((conf_iface_t*)n);
+		conf_free_iface((conf_iface_t*)n);
 	}
 	conf->ifaces_count = 0;
 	init_list(&conf->ifaces);
 
 	// Free logs
 	WALK_LIST_DELSAFE(n, nxt, conf->logs) {
-		log_free((conf_log_t*)n);
+		conf_free_log((conf_log_t*)n);
 	}
 	conf->logs_count = 0;
 	init_list(&conf->logs);
 
 	// Free remotes
 	WALK_LIST_DELSAFE(n, nxt, conf->remotes) {
-		iface_free((conf_iface_t*)n);
+		conf_free_iface((conf_iface_t*)n);
 	}
 	conf->remotes_count = 0;
 	init_list(&conf->remotes);
 
 	// Free zones
 	WALK_LIST_DELSAFE(n, nxt, conf->zones) {
-		zone_free((conf_zone_t*)n);
+		conf_free_zone((conf_zone_t*)n);
 	}
 	conf->zones_count = 0;
 	init_list(&conf->zones);
@@ -655,26 +642,28 @@ int conf_open(const char* path)
 {
 	/* Check path. */
 	if (!path) {
-		return KNOTD_EINVAL;
+		return KNOT_EINVAL;
 	}
 
 	/* Check if exists. */
 	struct stat st;
 	if (stat(path, &st) != 0) {
-		return KNOTD_ENOENT;
+		return KNOT_ENOENT;
 	}
 
 	/* Create new config. */
 	conf_t *nconf = conf_new(path);
 
 	/* Parse config. */
+	conf_parse_begin(nconf);
 	int ret = conf_fparser(nconf);
-	if (ret == KNOTD_EOK) {
+	conf_parse_end(nconf);
+	if (ret == KNOT_EOK) {
 		/* Postprocess config. */
 		ret = conf_process(nconf);
 	}
 	
-	if (ret != KNOTD_EOK) {
+	if (ret != KNOT_EOK) {
 		conf_free(nconf);
 		return ret;
 	}
@@ -703,7 +692,7 @@ int conf_open(const char* path)
 	/* Update hooks. */
 	conf_update_hooks(nconf);
 
-	return KNOTD_EOK;
+	return KNOT_EOK;
 }
 
 char* strcdup(const char *s1, const char *s2)
@@ -784,3 +773,63 @@ char* strcpath(char *path)
 	return path;
 }
 
+void conf_free_zone(conf_zone_t *zone)
+{
+	if (!zone) {
+		return;
+	}
+
+	/* Free ACL lists. */
+	WALK_LIST_FREE(zone->acl.xfr_in);
+	WALK_LIST_FREE(zone->acl.xfr_out);
+	WALK_LIST_FREE(zone->acl.notify_in);
+	WALK_LIST_FREE(zone->acl.notify_out);
+
+	free(zone->name);
+	free(zone->file);
+	free(zone->db);
+	free(zone->ixfr_db);
+	free(zone);
+}
+
+void conf_free_key(conf_key_t *k)
+{
+	/* Secure erase. */
+	if (k->k.secret) {
+		memset(k->k.secret, 0, strlen(k->k.secret));
+	}
+	free(k->k.secret);
+	knot_dname_free(&k->k.name);
+	free(k);
+}
+
+void conf_free_iface(conf_iface_t *iface)
+{
+	if (!iface) {
+		return;
+	}
+
+	free(iface->name);
+	free(iface->address);
+	free(iface);
+}
+
+void conf_free_log(conf_log_t *log)
+{
+	if (!log) {
+		return;
+	}
+
+	if (log->file) {
+		free(log->file);
+	}
+
+	/* Free loglevel mapping. */
+	node *n = 0, *nxt = 0;
+	WALK_LIST_DELSAFE(n, nxt, log->map) {
+		free((conf_log_map_t*)n);
+	}
+
+	free(log);
+}
+
