@@ -39,6 +39,7 @@
 #include "common/lists.h"
 #include "common/log.h"
 #include "common/sockaddr.h"
+#include "common/general-tree.h"
 
 /* Constants. */
 #define CONFIG_DEFAULT_PORT 53
@@ -96,6 +97,7 @@ typedef struct conf_zone_t {
 	int disable_any;          /*!< Disable ANY type queries for AA.*/
 	int notify_retries;       /*!< NOTIFY query retries. */
 	int notify_timeout;       /*!< Timeout for NOTIFY response (s). */
+	int build_diffs;          /*!< Calculate differences from changes. */
 	struct {
 		list xfr_in;      /*!< Remotes accepted for for xfr-in.*/
 		list xfr_out;     /*!< Remotes accepted for xfr-out.*/
@@ -197,7 +199,9 @@ typedef struct conf_t {
 	int notify_timeout; /*!< Timeout for NOTIFY response in seconds. */
 	int dbsync_timeout; /*!< Default interval between syncing to zonefile.*/
 	size_t ixfr_fslimit; /*!< File size limit for IXFR journal. */
-
+	int build_diffs;     /*!< Calculate differences from changes. */
+	general_tree_t *zone_tree; /*!< Zone tree for duplicate checking. */
+	
 	/*
 	 * Implementation specifics
 	 */
@@ -237,8 +241,8 @@ conf_t *conf_new(const char* path);
  * \param on_update Callback.
  * \param data User specified data for hook.
  *
- * \retval KNOTD_EOK on success.
- * \retval KNOTD_ENOMEM out of memory error.
+ * \retval KNOT_EOK on success.
+ * \retval KNOT_ENOMEM out of memory error.
  */
 int conf_add_hook(conf_t * conf, int sections,
                   int (*on_update)(const conf_t*, void*), void *data);
@@ -250,8 +254,8 @@ int conf_add_hook(conf_t * conf, int sections,
  *
  * \param conf Configuration context.
  *
- * \retval KNOTD_EOK on success.
- * \retval KNOTD_EPARSEFAIL on parser error.
+ * \retval KNOT_EOK on success.
+ * \retval KNOT_EPARSEFAIL on parser error.
  */
 int conf_parse(conf_t *conf);
 
@@ -263,8 +267,8 @@ int conf_parse(conf_t *conf);
  * \param conf Configuration context.
  * \param src Source string.
  *
- * \retval KNOTD_EOK on success.
- * \retval KNOTD_EPARSEFAIL on parser error.
+ * \retval KNOT_EOK on success.
+ * \retval KNOT_EPARSEFAIL on parser error.
  */
 int conf_parse_str(conf_t *conf, const char* src);
 
@@ -305,9 +309,9 @@ char* conf_find_default();
  *
  * \param path Path to configuration file.
  *
- * \retval KNOTD_EOK on success.
- * \retval KNOTD_EINVAL on null path.
- * \retval KNOTD_ENOENT if the path doesn't exist.
+ * \retval KNOT_EOK on success.
+ * \retval KNOT_EINVAL on null path.
+ * \retval KNOT_ENOENT if the path doesn't exist.
  */
 int conf_open(const char* path);
 
@@ -323,22 +327,6 @@ static inline conf_t* conf() {
 	return s_config; // Inline for performance reasons.
 }
 
-/*!
- * \brief Lock configuration for reading.
- *
- * \return Configuration context.
- */
-static inline void conf_read_lock() {
-	rcu_read_lock();
-}
-
-/*!
- * \brief Unlock configuration for reading.
- */
-static inline void conf_read_unlock() {
-	rcu_read_unlock();
-}
-
 /*
  * Utilities.
  */
@@ -363,6 +351,18 @@ char* strcdup(const char *s1, const char *s2);
  */
 char* strcpath(char *path);
 
+/*! \brief Free zone config. */
+void conf_free_zone(conf_zone_t *zone);
+
+/*! \brief Free TSIG key config. */
+void conf_free_key(conf_key_t *k);
+
+/*! \brief Free interface config. */
+void conf_free_iface(conf_iface_t *iface);
+
+/*! \brief Free log config. */
+void conf_free_log(conf_log_t *log);
+
 #endif /* _KNOTD_CONF_H_ */
 
 /*! @} */
