@@ -21,15 +21,17 @@
 #include <errno.h>
 #include <string.h>
 #include <signal.h>
+#include <grp.h>
+#include <unistd.h>
+#include <assert.h>
 
 #include "knot/common.h"
 #include "knot/ctl/process.h"
 #include "knot/conf/conf.h"
-#include "knot/other/error.h"
 
 char* pid_filename()
 {
-	conf_read_lock();
+	rcu_read_lock();
 
 	/* Read configuration. */
 	char* ret = 0;
@@ -37,7 +39,7 @@ char* pid_filename()
 		ret = strdup(conf()->pidfile);
 	}
 
-	conf_read_unlock();
+	rcu_read_unlock();
 
 	return ret;
 }
@@ -49,7 +51,7 @@ pid_t pid_read(const char* fn)
 	if (fn) {
 		FILE *fp = fopen(fn, "r");
 		if (!fp) {
-			return KNOTD_ENOENT;
+			return KNOT_ENOENT;
 		}
 
 		int readb = 0;
@@ -65,26 +67,26 @@ pid_t pid_read(const char* fn)
 
 		// Check read result
 		if (readb < 1) {
-			return KNOTD_ENOENT;
+			return KNOT_ENOENT;
 		}
 
 		// Convert pid
 		char* ep = 0;
 		unsigned long pid = strtoul(buf, &ep, 10);
 		if ((errno == ERANGE) || (*ep && !isspace(*ep))) {
-			return KNOTD_ERANGE;
+			return KNOT_ERANGE;
 		}
 
 		return (pid_t)pid;
 	}
 
-	return KNOTD_EINVAL;
+	return KNOT_EINVAL;
 }
 
 int pid_write(const char* fn)
 {
 	if (!fn) {
-		return KNOTD_EINVAL;
+		return KNOT_EINVAL;
 	}
 
 	// Convert
@@ -92,7 +94,7 @@ int pid_write(const char* fn)
 	int wbytes = 0;
 	wbytes = snprintf(buf, sizeof(buf), "%lu", (unsigned long) getpid());
 	if (wbytes < 0) {
-		return KNOTD_EINVAL;
+		return KNOT_EINVAL;
 	}
 
 	// Write
@@ -101,22 +103,23 @@ int pid_write(const char* fn)
 		int rc = fwrite(buf, wbytes, 1, fp);
 		fclose(fp);
 		if (rc < 0) {
-			return KNOTD_ERROR;
+			return KNOT_ERROR;
 		}
 
 		return 0;
 	}
 
-	return KNOTD_ENOENT;
+	return KNOT_ENOENT;
 }
 
 int pid_remove(const char* fn)
 {
 	if (unlink(fn) < 0) {
-		return KNOTD_EINVAL;
+		perror("unlink");
+		return KNOT_EINVAL;
 	}
 
-	return KNOTD_EOK;
+	return KNOT_EOK;
 }
 
 int pid_running(pid_t pid)
@@ -124,3 +127,45 @@ int pid_running(pid_t pid)
 	return kill(pid, 0) == 0;
 }
 
+void proc_update_privileges(int uid, int gid)
+{
+#ifdef HAVE_SETGROUPS
+	/* Drop supplementary groups. */
+	if (uid != getuid() || gid != getgid()) {
+		if (setgroups(0, NULL) < 0) {
+			log_server_warning("Failed to drop supplementary groups"
+			                   " for uid '%d' (%s).\n",
+			                   getuid(), strerror(errno));
+		}
+	}
+#endif
+	
+	/* Watch uid/gid. */
+	if (gid != getgid()) {
+		log_server_info("Changing group id to '%d'.\n", gid);
+		if (setregid(gid, gid) < 0) {
+			log_server_error("Failed to change gid to '%d'.\n",
+			                 gid);
+		}
+	}
+	if (uid != getuid()) {
+		log_server_info("Changing user id to '%d'.\n", uid);
+		if (setreuid(uid, uid) < 0) {
+			log_server_error("Failed to change uid to '%d'.\n",
+			                 uid);
+		}
+	}
+	
+	/* Check storage writeability. */
+	char *lfile = strcdup(conf()->storage, "/knot.lock");
+	assert(lfile != NULL);
+	FILE* fp = fopen(lfile, "w");
+	if (fp == NULL) {
+		log_server_warning("Storage directory '%s' is not writeable.\n",
+		                   conf()->storage);
+	} else {
+		fclose(fp);
+		unlink(lfile);
+	}
+	free(lfile);
+}
