@@ -20,6 +20,7 @@
 #include <unistd.h>
 #include <getopt.h>
 #include <limits.h>
+
 #ifdef HAVE_CAP_NG_H
 #include <cap-ng.h>
 #endif /* HAVE_CAP_NG_H */
@@ -27,7 +28,6 @@
 #include "common.h"
 #include "common/evqueue.h"
 #include "knot/common.h"
-#include "knot/other/error.h"
 #include "knot/server/server.h"
 #include "knot/ctl/process.h"
 #include "knot/conf/conf.h"
@@ -72,7 +72,7 @@ void help(int argc, char **argv)
 {
 	printf("Usage: %sd [parameters]\n",
 	       PACKAGE_NAME);
-	printf("Parameters:\n"
+	printf("\nParameters:\n"
 	       " -c, --config [file] Select configuration file.\n"
 	       " -d, --daemonize     Run server as a daemon.\n"
 	       " -v, --verbose       Verbose mode - additional runtime information.\n"
@@ -138,6 +138,7 @@ int main(int argc, char **argv)
 	emptyset.sa_flags = 0;
 	sigaction(SIGALRM, &emptyset, NULL); // Interrupt
 	sigaction(SIGPIPE, &emptyset, NULL); // Mask
+	rcu_register_thread();
 
 	// Setup event queue
 	evqueue_set(evqueue_new());
@@ -158,11 +159,10 @@ int main(int argc, char **argv)
 	server_t *server = server_create();
 
 	// Initialize configuration
-	conf_read_lock();
+	rcu_read_lock();
 	conf_add_hook(conf(), CONF_LOG, log_conf_hook, 0);
 	conf_add_hook(conf(), CONF_ALL, server_conf_hook, server);
-	conf_add_hook(conf(), CONF_ALL, zones_ns_conf_hook, server->nameserver);
-	conf_read_unlock();
+	rcu_read_unlock();
 
 	// Find implicit configuration file
 	if (!config_fn) {
@@ -224,12 +224,12 @@ int main(int argc, char **argv)
 	// Open configuration
 	log_server_info("Reading configuration '%s' ...\n", config_fn);
 	int conf_ret = conf_open(config_fn);
-	if (conf_ret != KNOTD_EOK) {
-		if (conf_ret == KNOTD_ENOENT) {
+	if (conf_ret != KNOT_EOK) {
+		if (conf_ret == KNOT_ENOENT) {
 			log_server_error("Couldn't open configuration file "
 					 "'%s'.\n", config_fn);
 		} else {
-			log_server_error("Failed to parse configuration '%s'.\n",
+			log_server_error("Failed to load configuration '%s'.\n",
 				config_fn);
 		}
 		server_wait(server);
@@ -242,21 +242,28 @@ int main(int argc, char **argv)
 	}
 	log_server_info("\n");
 
-	// Create server instance
-	char* pidfile = pid_filename();
+	/* Alter privileges. */
+	proc_update_privileges(conf()->uid, conf()->gid);
+	
+	/* Load zones and add hook. */
+	zones_ns_conf_hook(conf(), server->nameserver);
+	conf_add_hook(conf(), CONF_ALL, zones_ns_conf_hook, server->nameserver);
 
 	// Run server
 	int res = 0;
+	int has_pid = 0;
+	char* pidfile = pid_filename();
 	log_server_info("Starting server...\n");
-	if ((server_start(server)) == KNOTD_EOK) {
+	if ((server_start(server)) == KNOT_EOK) {
 
 		// Save PID
-		int has_pid = 1;
+		has_pid = 1;
 		int rc = pid_write(pidfile);
 		if (rc < 0) {
 			has_pid = 0;
 			log_server_warning("Failed to create "
-					   "PID file '%s'.\n", pidfile);
+					   "PID file '%s' (%s).\n",
+					   pidfile, strerror(errno));
 		}
 
 		// Change directory if daemonized
@@ -310,11 +317,11 @@ int main(int argc, char **argv)
 				sig_req_reload = 0;
 				int cf_ret = conf_open(config_fn);
 				switch (cf_ret) {
-				case KNOTD_EOK:
+				case KNOT_EOK:
 					log_server_info("Configuration "
 							"reloaded.\n");
 					break;
-				case KNOTD_ENOENT:
+				case KNOT_ENOENT:
 					log_server_error("Configuration "
 							 "file '%s' "
 							 "not found.\n",
@@ -330,10 +337,10 @@ int main(int argc, char **argv)
 				log_server_info("Refreshing slave zones...\n");
 				sig_req_reload = 0;
 				int cf_ret = server_refresh(server);
-				if (cf_ret != KNOTD_EOK) {
+				if (cf_ret != KNOT_EOK) {
 					log_server_error("Couldn't refresh "
 					                 "slave zones - %s",
-					                 knotd_strerror(cf_ret));
+					                 knot_strerror(cf_ret));
 				}
 				
 			}
@@ -352,7 +359,7 @@ int main(int argc, char **argv)
 		}
 		pthread_sigmask(SIG_UNBLOCK, &sa.sa_mask, NULL);
 
-		if ((server_wait(server)) != KNOTD_EOK) {
+		if ((server_wait(server)) != KNOT_EOK) {
 			log_server_error("An error occured while "
 					 "waiting for server to finish.\n");
 			res = 1;
@@ -370,7 +377,7 @@ int main(int argc, char **argv)
 	server_destroy(&server);
 
 	// Remove PID file
-	if (pid_remove(pidfile) < 0) {
+	if (has_pid && pid_remove(pidfile) < 0) {
 		log_server_warning("Failed to remove PID file.\n");
 	}
 
@@ -381,6 +388,8 @@ int main(int argc, char **argv)
 	// Destroy event loop
 	evqueue_t *q = evqueue();
 	evqueue_free(&q);
+	
+	rcu_unregister_thread();
 
 	// Free default config filename if exists
 	free(config_fn);
