@@ -21,14 +21,23 @@
 #include <stdio.h>
 #include <unistd.h>
 #include <errno.h>
+#include <urcu.h>
+
 #ifdef HAVE_CAP_NG_H
 #include <cap-ng.h>
 #endif /* HAVE_CAP_NG_H */
+#ifdef HAVE_PTHREAD_NP_H
+#include <pthread_np.h>
+#endif /* HAVE_PTHREAD_NP_H */
 
 #include "knot/common.h"
 #include "knot/server/dthreads.h"
 #include "common/log.h"
-#include "knot/other/error.h"
+
+/* BSD cpu set compatibility. */
+#if defined(HAVE_CPUSET_BSD)
+typedef cpuset_t cpu_set_t;
+#endif
 
 /*! \brief Lock thread state for R/W. */
 static inline void lock_thread_rw(dthread_t *thread)
@@ -60,13 +69,13 @@ static inline int dt_update_thread(dthread_t *thread, int state)
 {
 	// Check
 	if (thread == 0) {
-		return KNOTD_EINVAL;
+		return KNOT_EINVAL;
 	}
 
 	// Cancel with lone thread
 	dt_unit_t *unit = thread->unit;
 	if (unit == 0) {
-		return KNOTD_ENOTSUP;
+		return KNOT_ENOTSUP;
 	}
 
 	// Cancel current runnable if running
@@ -86,10 +95,10 @@ static inline int dt_update_thread(dthread_t *thread, int state)
 		/* Unable to update thread, it is already dead. */
 		unlock_thread_rw(thread);
 		pthread_mutex_unlock(&unit->_notify_mx);
-		return KNOTD_EINVAL;
+		return KNOT_EINVAL;
 	}
 
-	return KNOTD_EOK;
+	return KNOT_EOK;
 }
 
 /*!
@@ -126,6 +135,7 @@ static void *thread_ep(void *data)
 	sigaddset(&ignset, SIGPIPE);
 	sigaddset(&ignset, SIGUSR1);
 	pthread_sigmask(SIG_BLOCK, &ignset, 0); /*! \todo Review under BSD (issue #1441). */
+	rcu_register_thread();
 
 	dbg_dt("dthreads: [%p] entered ep\n", thread);
 	
@@ -207,6 +217,7 @@ static void *thread_ep(void *data)
 	lock_thread_rw(thread);
 	thread->state |= ThreadJoinable;
 	unlock_thread_rw(thread);
+	rcu_unregister_thread();
 
 	// Return
 	return 0;
@@ -424,6 +435,7 @@ void dt_delete(dt_unit_t **unit)
 	// Deinit mutexes
 	pthread_mutex_destroy(&d_unit->_notify_mx);
 	pthread_mutex_destroy(&d_unit->_report_mx);
+	pthread_mutex_destroy(&d_unit->_mx);
 
 	// Deinit conditions
 	pthread_cond_destroy(&d_unit->_notify);
@@ -439,7 +451,7 @@ int dt_resize(dt_unit_t *unit, int size)
 {
 	// Check input
 	if (unit == 0 || size <= 0) {
-		return KNOTD_EINVAL;
+		return KNOT_EINVAL;
 	}
 
 	// Evaluate delta
@@ -493,7 +505,7 @@ int dt_resize(dt_unit_t *unit, int size)
 	// New threads vector
 	dthread_t **threads = malloc(size * sizeof(dthread_t *));
 	if (threads == 0) {
-		return KNOTD_ENOMEM;
+		return KNOT_ENOMEM;
 	}
 
 	// Lock unit
@@ -611,7 +623,7 @@ int dt_start(dt_unit_t *unit)
 {
 	// Check input
 	if (unit == 0) {
-		return KNOTD_EINVAL;
+		return KNOT_EINVAL;
 	}
 
 	// Lock unit
@@ -643,7 +655,7 @@ int dt_start_id(dthread_t *thread)
 {
 	// Check input
 	if (thread == 0) {
-		return KNOTD_EINVAL;
+		return KNOT_EINVAL;
 	}
 
 	lock_thread_rw(thread);
@@ -679,29 +691,29 @@ int dt_signalize(dthread_t *thread, int signum)
 {
 	// Check input
 	if (thread == 0) {
-		return KNOTD_EINVAL;
+		return KNOT_EINVAL;
 	}
 
 	int ret = pthread_kill(thread->_thr, signum);
 
 	/* Not thread id found or invalid signum. */
 	if (ret == EINVAL || ret == ESRCH) {
-		return KNOTD_EINVAL;
+		return KNOT_EINVAL;
 	}
 
 	/* Generic error. */
 	if (ret < 0) {
-		return KNOTD_ERROR;
+		return KNOT_ERROR;
 	}
 
-	return KNOTD_EOK;
+	return KNOT_EOK;
 }
 
 int dt_join(dt_unit_t *unit)
 {
 	// Check input
 	if (unit == 0) {
-		return KNOTD_EINVAL;
+		return KNOT_EINVAL;
 	}
 
 	for (;;) {
@@ -751,14 +763,14 @@ int dt_join(dt_unit_t *unit)
 		pthread_mutex_unlock(&unit->_report_mx);
 	}
 
-	return KNOTD_EOK;
+	return KNOT_EOK;
 }
 
 int dt_stop_id(dthread_t *thread)
 {
 	// Check input
 	if (thread == 0) {
-		return KNOTD_EINVAL;
+		return KNOT_EINVAL;
 	}
 
 	// Signalize active thread to stop
@@ -777,14 +789,14 @@ int dt_stop_id(dthread_t *thread)
 		pthread_mutex_unlock(&unit->_notify_mx);
 	}
 
-	return KNOTD_EOK;
+	return KNOT_EOK;
 }
 
 int dt_stop(dt_unit_t *unit)
 {
 	// Check unit
 	if (unit == 0) {
-		return KNOTD_EINVAL;
+		return KNOT_EINVAL;
 	}
 
 	// Lock unit
@@ -813,14 +825,14 @@ int dt_stop(dt_unit_t *unit)
 	pthread_cond_broadcast(&unit->_notify);
 	pthread_mutex_unlock(&unit->_notify_mx);
 
-	return KNOTD_EOK;
+	return KNOT_EOK;
 }
 
 //int dt_setprio(dthread_t *thread, int prio)
 //{
 //	// Check input
 //	if (thread == 0) {
-//		return KNOTD_EINVAL;
+//		return KNOT_EINVAL;
 //	}
 
 //	// Clamp priority
@@ -845,42 +857,64 @@ int dt_stop(dt_unit_t *unit)
 
 //		/* Map "not supported". */
 //		if (errno == ENOTSUP) {
-//			return KNOTD_ENOTSUP;
+//			return KNOT_ENOTSUP;
 //		}
 
-//		return KNOTD_EINVAL;
+//		return KNOT_EINVAL;
 //	}
 
-//	return KNOTD_EOK;
+//	return KNOT_EOK;
 //}
 
-int dt_setaffinity(dthread_t *thread, void *mask, size_t len)
+
+
+int dt_setaffinity(dthread_t *thread, unsigned* cpu_id, size_t cpu_count)
 {
-	if (thread == NULL || mask == NULL) {
-		return KNOTD_EINVAL;
+	if (thread == NULL) {
+		return KNOT_EINVAL;
 	}
 	
 #ifdef HAVE_PTHREAD_SETAFFINITY_NP
-	if (len != sizeof(cpu_set_t)) {
-		return KNOTD_EINVAL;
+	int ret = -1;
+	
+/* Linux, FreeBSD interface. */
+#if defined(HAVE_CPUSET_LINUX) || defined(HAVE_CPUSET_BSD)
+	cpu_set_t set;
+	CPU_ZERO(&set);
+	for (unsigned i = 0; i < cpu_count; ++i) {
+		CPU_SET(cpu_id[i], &set);
 	}
-	pthread_t tid = pthread_self();
-	int ret = pthread_setaffinity_np(tid, len, (cpu_set_t*)mask);
+	ret = pthread_setaffinity_np(thread->_thr, sizeof(cpu_set_t), &set);
+/* NetBSD interface. */
+#elif defined(HAVE_CPUSET_NETBSD)
+	cpuset_t *set = cpuset_create();
+	if (set == NULL) {
+		return KNOT_ENOMEM;
+	}
+	cpuset_zero(set);
+	for (unsigned i = 0; i < cpu_count; ++i) {
+		cpuset_set(cpu_id[i], &set);
+	}
+	ret = pthread_setaffinity_np(thread->_thr, cpuset_size(set), set);
+	cpuset_destroy(set);
+#endif /* interface */
+
 	if (ret < 0) {
-		return KNOTD_ERROR;
+		return KNOT_ERROR;
 	}
-#else
-	return KNOTD_ENOTSUP;
+
+#else /* HAVE_PTHREAD_SETAFFINITY_NP */
+	return KNOT_ENOTSUP;
 #endif
 	
-	return KNOTD_EOK;
+	return KNOT_EOK;
 }
 
 int dt_repurpose(dthread_t *thread, runnable_t runnable, void *data)
 {
 	// Check
 	if (thread == 0) {
-		return KNOTD_EINVAL;
+		return KNOT_EINVAL;
 	}
 
 	// Stop here if thread isn't a member of a unit
@@ -889,7 +923,7 @@ int dt_repurpose(dthread_t *thread, runnable_t runnable, void *data)
 		lock_thread_rw(thread);
 		thread->state = ThreadActive | ThreadCancelled;
 		unlock_thread_rw(thread);
-		return KNOTD_ENOTSUP;
+		return KNOT_ENOTSUP;
 	}
 
 	// Lock thread state changes
@@ -915,7 +949,7 @@ int dt_repurpose(dthread_t *thread, runnable_t runnable, void *data)
 		pthread_mutex_unlock(&unit->_notify_mx);
 	}
 
-	return KNOTD_EOK;
+	return KNOT_EOK;
 }
 
 int dt_activate(dthread_t *thread)
@@ -932,7 +966,7 @@ int dt_compact(dt_unit_t *unit)
 {
 	// Check input
 	if (unit == 0) {
-		return KNOTD_EINVAL;
+		return KNOT_EINVAL;
 	}
 
 	// Lock unit
@@ -982,14 +1016,23 @@ int dt_compact(dt_unit_t *unit)
 	// Unlock unit
 	dt_unit_unlock(unit);
 
-	return KNOTD_EOK;
+	return KNOT_EOK;
 }
 
 int dt_online_cpus()
 {
 	int ret = -1;
+/* Linux, Solaris, OS X 10.4+ */
 #ifdef _SC_NPROCESSORS_ONLN
 	ret = (int) sysconf(_SC_NPROCESSORS_ONLN);
+#else
+/* FreeBSD, NetBSD, OpenBSD, OS X < 10.4 */
+#if HAVE_SYSCTLBYNAME
+	size_t rlen = sizeof(int);
+	if (sysctlbyname("hw.ncpu", &ret, &rlen, NULL, 0) < 0) {
+		ret = -1;
+	}
+#endif
 #endif
 	return ret;
 }
@@ -1043,7 +1086,7 @@ int dt_unit_lock(dt_unit_t *unit)
 {
 	// Check input
 	if (unit == 0) {
-		return KNOTD_EINVAL;
+		return KNOT_EINVAL;
 	}
 
 	int ret = pthread_mutex_lock(&unit->_mx);
@@ -1053,14 +1096,14 @@ int dt_unit_lock(dt_unit_t *unit)
 		return knot_map_errno(EINVAL, EAGAIN);
 	}
 
-	return KNOTD_EOK;
+	return KNOT_EOK;
 }
 
 int dt_unit_unlock(dt_unit_t *unit)
 {
 	// Check input
 	if (unit == 0) {
-		return KNOTD_EINVAL;
+		return KNOT_EINVAL;
 	}
 
 	int ret = pthread_mutex_unlock(&unit->_mx);
@@ -1070,5 +1113,5 @@ int dt_unit_unlock(dt_unit_t *unit)
 		return knot_map_errno(EINVAL, EAGAIN);
 	}
 
-	return KNOTD_EOK;
+	return KNOT_EOK;
 }
