@@ -22,12 +22,9 @@
 #include <errno.h>
 #include <openssl/evp.h>
 #include <assert.h>
-#include <grp.h>
-
 
 #include "common/prng.h"
 #include "knot/common.h"
-#include "knot/other/error.h"
 #include "knot/server/server.h"
 #include "knot/server/udp-handler.h"
 #include "knot/server/tcp-handler.h"
@@ -45,7 +42,7 @@ static int evsched_run(dthread_t *thread)
 	iohandler_t *sched_h = (iohandler_t *)thread->data;
 	evsched_t *s = (evsched_t*)sched_h->data;
 	if (!s) {
-		return KNOTD_EINVAL;
+		return KNOT_EINVAL;
 	}
 
 	/* Run event loop. */
@@ -74,7 +71,7 @@ static int evsched_run(dthread_t *thread)
 		}
 	}
 
-	return KNOTD_EOK;
+	return KNOT_EOK;
 }
 
 /*! \brief List item for generic pointers. */
@@ -203,7 +200,7 @@ static int server_init_iface(iface_t *new_if, conf_iface_t *cfg_if)
 	new_if->type[TCP_ID] = cfg_if->family;
 	new_if->port = cfg_if->port;
 	new_if->addr = strdup(cfg_if->address);
-	return KNOTD_EOK;
+	return KNOT_EOK;
 }
 
 /*!
@@ -219,7 +216,7 @@ static int server_bind_sockets(server_t *server)
 	 */
 
 	/* Lock configuration. */
-	conf_read_lock();
+	rcu_read_lock();
 
 	/* Prepare helper lists. */
 	int bound = 0;
@@ -270,6 +267,8 @@ static int server_bind_sockets(server_t *server)
 		if (found_match) {
 			rem_node(m);
 		} else {
+			log_server_info("Binding to interface %s port %d.\n",
+			                cfg_if->address, cfg_if->port);
 
 			/* Create new interface. */
 			m = malloc(sizeof(iface_t));
@@ -277,9 +276,6 @@ static int server_bind_sockets(server_t *server)
 				free(m);
 				m = 0;
 			}
-
-			log_server_info("Binding to interface %s port %d.\n",
-			                cfg_if->address, cfg_if->port);
 		}
 
 		/* Move to new list. */
@@ -290,7 +286,7 @@ static int server_bind_sockets(server_t *server)
 	}
 
 	/* Unlock configuration. */
-	conf_read_unlock();
+	rcu_read_unlock();
 
 	/* Publish new list. */
 	list* oldlist = rcu_xchg_pointer(&server->ifaces, newlist);
@@ -327,11 +323,11 @@ static int server_bind_sockets(server_t *server)
 static int server_bind_handlers(server_t *server)
 {
 	if (!server || !server->ifaces) {
-		return KNOTD_EINVAL;
+		return KNOT_EINVAL;
 	}
 	
 	/* Lock config. */
-	conf_read_lock();
+	rcu_read_lock();
 
 	/* Estimate number of threads/manager. */
 	int thr_count = 0;
@@ -388,9 +384,9 @@ static int server_bind_handlers(server_t *server)
 	}
 
 	/* Unlock config. */
-	conf_read_unlock();
+	rcu_read_unlock();
 
-	return KNOTD_EOK;
+	return KNOT_EOK;
 }
 
 server_t *server_create()
@@ -484,7 +480,7 @@ int server_remove_handler(server_t *server, iohandler_t *h)
 {
 	// Check
 	if (h == 0) {
-		return KNOTD_EINVAL;
+		return KNOT_EINVAL;
 	}
 
 	/* Lock RCU. */
@@ -536,14 +532,14 @@ int server_remove_handler(server_t *server, iohandler_t *h)
 	// Destroy dispatcher and worker
 	dt_delete(&h->unit);
 	free(h);
-	return KNOTD_EOK;
+	return KNOT_EOK;
 }
 
 int server_start(server_t *server)
 {
 	// Check server
 	if (server == 0) {
-		return KNOTD_EINVAL;
+		return KNOT_EINVAL;
 	}
 
 	dbg_server("server: starting server instance\n");
@@ -552,10 +548,10 @@ int server_start(server_t *server)
 	xfr_start(server->xfr_h);
 
 	/* Lock configuration. */
-	conf_read_lock();
+	rcu_read_lock();
 
 	// Start dispatchers
-	int ret = KNOTD_EOK;
+	int ret = KNOT_EOK;
 	server->state |= ServerRunning;
 	iohandler_t *h = 0;
 	WALK_LIST(h, server->handlers) {
@@ -573,7 +569,7 @@ int server_start(server_t *server)
 	}
 
 	/* Unlock configuration. */
-	conf_read_unlock();
+	rcu_read_unlock();
 
 	dbg_server("server: server started\n");
 
@@ -616,7 +612,7 @@ int server_wait(server_t *server)
 int server_refresh(server_t *server)
 {
 	if (server == NULL || server->nameserver == NULL) {
-		return KNOTD_EINVAL;
+		return KNOT_EINVAL;
 	}
 	
 	/* Lock RCU and fetch zones. */
@@ -626,7 +622,7 @@ int server_refresh(server_t *server)
 	const knot_zone_t **zones = knot_zonedb_zones(ns->zone_db);
 	if (zones == NULL) {
 		rcu_read_unlock();
-		return KNOTD_ENOMEM;
+		return KNOT_ENOMEM;
 	}
 	
 	/* REFRESH zones. */
@@ -646,23 +642,20 @@ int server_refresh(server_t *server)
 	/* Unlock RCU. */
 	rcu_read_unlock();
 	free(zones);
-	return KNOTD_EOK;
+	return KNOT_EOK;
 }
 
 void server_stop(server_t *server)
 {
 	dbg_server("server: stopping server\n");
 	
-	/* Wait for XFR master. */
-	xfr_stop(server->xfr_h);
+	/* Send termination event. */
+	evsched_schedule_term(server->sched, 0);
 
 	/* Interrupt XFR handler execution. */
 	if (server->xfr_h->interrupt) {
 		server->xfr_h->interrupt(server->xfr_h);
 	}
-	
-	/* Send termination event. */
-	evsched_schedule_term(server->sched, 0);
 
 	/* Lock RCU. */
 	rcu_read_lock();
@@ -728,11 +721,11 @@ int server_conf_hook(const struct conf_t *conf, void *data)
 	server_t *server = (server_t *)data;
 
 	if (!server) {
-		return KNOTD_EINVAL;
+		return KNOT_EINVAL;
 	}
 
 	/* Update bound sockets. */
-	int ret = KNOTD_EOK;
+	int ret = KNOT_EOK;
 	if ((ret = server_bind_sockets(server)) < 0) {
 		log_server_error("Failed to bind configured "
 		                 "interfaces.\n");
@@ -743,52 +736,10 @@ int server_conf_hook(const struct conf_t *conf, void *data)
 			                 "configured interfaces.\n");
 		}
 	}
-	
-	/* Lock configuration. */
-	conf_read_lock();
-	int priv_failed = 0;
-	
-#ifdef HAVE_SETGROUPS
-	/* Drop supplementary groups. */
-	if (conf->gid > -1 || conf->uid > -1) {
-		ret = setgroups(0, NULL);
-		
-		/* Collect results. */
-		if (ret < 0) {
-			log_server_error("Failed to set supplementary groups "
-			                 "for uid '%d' (%s).\n",
-			                 getuid(), strerror(errno));
-			priv_failed = 1;
-		}
-	}
-#endif
-	
-	/* Watch uid/gid. */
-	if (conf->gid > -1 && conf->gid != getgid()) {
-		log_server_info("Changing group id to '%d'.\n", conf->gid);
-		if (setregid(conf->gid, conf->gid) < 0) {
-			log_server_error("Failed to change gid to '%d'.\n",
-			                 conf->gid);
-			priv_failed = 1;
-		}
-	}
-	if (conf->uid > -1 && conf->uid != getuid()) {
-		log_server_info("Changing user id to '%d'.\n", conf->uid);
-		if (setreuid(conf->uid, conf->uid) < 0) {
-			log_server_error("Failed to change uid to '%d'.\n",
-			                 conf->uid);
-			priv_failed = 1;
-		}
-	}
-
-	if (priv_failed) {
-		ret = KNOTD_EACCES;
-	}
 
 	/* Exit if the server is not running. */
-	if (ret != KNOTD_EOK || !(server->state & ServerRunning)) {
-		conf_read_unlock();
-		return KNOTD_ENOTRUNNING;
+	if (ret != KNOT_EOK || !(server->state & ServerRunning)) {
+		return KNOT_ENOTRUNNING;
 	}
 
 	/* Start new handlers. */
@@ -807,9 +758,6 @@ int server_conf_hook(const struct conf_t *conf, void *data)
 		}
 	}
 
-	/* Unlock config. */
-	conf_read_unlock();
-
 	return ret;
 }
 
