@@ -22,12 +22,15 @@
 #include <errno.h>
 #include <stdio.h>
 #include <netdb.h>
+#include <time.h>
 #include <sys/socket.h>
+#ifdef HAVE_NETINET_IN_SYSTM_H
+#include <netinet/in_systm.h>
+#endif
 #include <netinet/in.h>
 #include <netinet/ip.h>
 #include <arpa/inet.h>
 
-#include "knot/other/error.h"
 #include "knot/common.h"
 #include "knot/server/socket.h"
 
@@ -50,12 +53,12 @@ int socket_connect(int fd, const char *addr, unsigned short port)
 	}
 
 	/* Resolve address. */
-	int ret = KNOTD_EOK;
+	int ret = KNOT_EOK;
 	struct addrinfo hints, *res;
 	hints.ai_family = AF_UNSPEC;
 	hints.ai_socktype = SOCK_STREAM;
 	if ((ret = getaddrinfo(addr, NULL, &hints, &res)) != 0) {
-		return KNOTD_EINVAL;
+		return KNOT_EINVAL;
 	}
 
 	/* Evaluate address type. */
@@ -95,6 +98,8 @@ int socket_connect(int fd, const char *addr, unsigned short port)
 int socket_bind(int socket, int family, const char *addr, unsigned short port)
 {
 	/* Check address family. */
+	int flag = 1;
+	int ret = 0;
 	struct sockaddr* paddr = 0;
 	socklen_t addrlen = 0;
 	struct sockaddr_in saddr;
@@ -107,7 +112,7 @@ int socket_bind(int socket, int family, const char *addr, unsigned short port)
 		paddr = (struct sockaddr*)&saddr;
 		addrlen = sizeof(saddr);
 		if (getsockname(socket, paddr, &addrlen) < 0) {
-			return KNOTD_EINVAL;
+			return KNOT_EINVAL;
 		}
 
 		/* Set address and port. */
@@ -126,13 +131,13 @@ int socket_bind(int socket, int family, const char *addr, unsigned short port)
 
 #ifdef DISABLE_IPV6
 		log_server_error("ipv6 support disabled\n");
-		return KNOTD_ENOIPV6;
+		return KNOT_ENOIPV6;
 #else
 		/* Initialize socket address. */
 		paddr = (struct sockaddr*)&saddr6;
 		addrlen = sizeof(saddr6);
 		if (getsockname(socket, paddr, &addrlen) < 0) {
-			return KNOTD_EINVAL;
+			return KNOT_EINVAL;
 		}
 
 		/* Set address and port. */
@@ -146,26 +151,37 @@ int socket_bind(int socket, int family, const char *addr, unsigned short port)
 			                 addr, buf);
 
 		}
-#endif
+
+		/* Make the socket IPv6 only to allow 'any' for IPv4 and IPv6 at the same time. */
+#ifdef IPV6_V6ONLY
+		if (family == AF_INET6) {
+			/* Do not support mapping IPv4 in IPv6 sockets. */
+			ret = setsockopt(socket, IPPROTO_IPV6, IPV6_V6ONLY,
+			                 &flag, sizeof(flag));
+			if (ret < 0) {
+				return KNOT_EINVAL;
+			}
+		}
+#endif /* IPV6_V6ONLY */
+#endif /* DISABLE_IPV6 */
 	}
 
 	/* Reuse old address if taken. */
-	int flag = 1;
-	int ret = setsockopt(socket, SOL_SOCKET, SO_REUSEADDR,
+	ret = setsockopt(socket, SOL_SOCKET, SO_REUSEADDR,
 	                     &flag, sizeof(flag));
 	if (ret < 0) {
-		return KNOTD_EINVAL;
+		return KNOT_EINVAL;
 	}
 
 	/* Bind to specified address. */
 	int res = bind(socket, paddr, addrlen);
 	if (res < 0) {
-		log_server_error("Cannot bind to socket (%d).\n",
+		log_server_error("Cannot bind to socket (errno %d).\n",
 		                 errno);
 		return knot_map_errno(EADDRINUSE, EINVAL, EACCES, ENOMEM);
 	}
 
-	return KNOTD_EOK;
+	return KNOT_EOK;
 }
 
 int socket_listen(int socket, int backlog_size)
@@ -175,15 +191,15 @@ int socket_listen(int socket, int backlog_size)
 		return knot_map_errno(EADDRINUSE);
 	}
 
-	return KNOTD_EOK;
+	return KNOT_EOK;
 }
 
 int socket_close(int socket)
 {
 	if (close(socket) < 0) {
-		return KNOTD_EINVAL;
+		return KNOT_EINVAL;
 	}
 
-	return KNOTD_EOK;
+	return KNOT_EOK;
 }
 
