@@ -37,7 +37,6 @@
 
 #include "common/sockaddr.h"
 #include "knot/common.h"
-#include "knot/other/error.h"
 #include "knot/server/udp-handler.h"
 #include "libknot/nameserver/name-server.h"
 #include "knot/stat/stat.h"
@@ -47,7 +46,6 @@
 #include "libknot/packet/packet.h"
 #include "knot/server/zones.h"
 #include "knot/server/notify.h"
-#include "libknot/util/error.h"
 
 /* Check for sendmmsg syscall. */
 #ifdef HAVE_SENDMMSG
@@ -85,51 +83,53 @@ int udp_handle(int fd, uint8_t *qbuf, size_t qbuflen, size_t *resp_len,
 	if (packet == NULL) {
 		dbg_net("udp: failed to create packet on fd=%d\n", fd);
 
-		int ret = knot_ns_error_response_from_query(ns, qbuf, qbuflen,
+		int ret = knot_ns_error_response_from_query_wire(ns, qbuf, qbuflen,
 		                                            KNOT_RCODE_SERVFAIL,
 		                                            qbuf, resp_len);
 
 		if (ret != KNOT_EOK) {
-			return KNOTD_EMALF;
+			return KNOT_EMALF;
 		}
 
-		return KNOTD_EOK; /* Created error response. */
+		return KNOT_EOK; /* Created error response. */
 	}
 
 	/* Parse query. */
 	int res = knot_ns_parse_packet(qbuf, qbuflen, packet, &qtype);
-	if (unlikely(res != KNOTD_EOK)) {
+	if (knot_unlikely(res != KNOT_EOK)) {
 		dbg_net("udp: failed to parse packet on fd=%d\n", fd);
 		if (res > 0) { /* Returned RCODE */
-			int ret = knot_ns_error_response_from_query(ns, qbuf,
-			                                            qbuflen,
-			                                            res, qbuf,
-			                                            resp_len);
+//			int ret = knot_ns_error_response_from_query_wire(ns,
+//				qbuf, qbuflen, res, qbuf, resp_len);
+			int ret = knot_ns_error_response_from_query(ns,
+				packet, res, qbuf, resp_len);
 
 			if (ret != KNOT_EOK) {
 				knot_packet_free(&packet);
-				return KNOTD_EMALF;
+				return KNOT_EMALF;
+			}
+		} else {
+			assert(res < 0);
+			int ret = knot_ns_error_response_from_query_wire(
+			       ns, qbuf, qbuflen, KNOT_RCODE_SERVFAIL, qbuf, 
+			       resp_len);
+			
+			if (ret != KNOT_EOK) {
+				knot_packet_free(&packet);
+				return ret;
 			}
 		}
 
 		knot_packet_free(&packet);
-		return KNOTD_EOK; /* Created error response. */
+		return KNOT_EOK; /* Created error response. */
 	}
 
 	/* Handle query. */
 //	server_t *srv = (server_t *)knot_ns_get_data(ns);
 //	knot_ns_xfr_t xfr;
-	res = KNOTD_ERROR;
+	res = KNOT_ERROR;
 	switch(qtype) {
 
-	/* Response types. */
-	case KNOT_RESPONSE_NORMAL:
-		res = zones_process_response(ns, addr, packet, qbuf, resp_len);
-		break;
-	case KNOT_RESPONSE_NOTIFY:
-		res = notify_process_response(ns, packet, addr, qbuf, resp_len);
-		break;
-	
 	/* Query types. */
 	case KNOT_QUERY_NORMAL:
 		res = zones_normal_query_answer(ns, packet, addr, qbuf,
@@ -140,10 +140,10 @@ int udp_handle(int fd, uint8_t *qbuf, size_t qbuflen, size_t *resp_len,
 		 * Bind responds with FORMERR.
  		 */
 		/*! \note Draft exists for AXFR/UDP, but has not been standardized. */
-		knot_ns_error_response(ns, knot_packet_id(packet),
-		                       &packet->header.flags1,
-		                       KNOT_RCODE_FORMERR, qbuf, resp_len);
-		res = KNOTD_EOK;
+		knot_ns_error_response_from_query(ns, packet,
+		                                  KNOT_RCODE_FORMERR, qbuf,
+		                                  resp_len);
+		res = KNOT_EOK;
 		break;
 	case KNOT_QUERY_IXFR:
 		/* According to RFC1035, respond with SOA. 
@@ -151,7 +151,7 @@ int udp_handle(int fd, uint8_t *qbuf, size_t qbuflen, size_t *resp_len,
 		 * but I have found no tool or slave server to actually attempt
 		 * IXFR/UDP.
 		 */
-		knot_packet_set_qtype(packet, KNOT_RRTYPE_SOA);
+//		knot_packet_set_qtype(packet, KNOT_RRTYPE_SOA);
 		res = zones_normal_query_answer(ns, packet, addr,
 		                                qbuf, resp_len, 
 		                                NS_TRANSPORT_UDP);
@@ -163,28 +163,27 @@ int udp_handle(int fd, uint8_t *qbuf, size_t qbuflen, size_t *resp_len,
 		
 	case KNOT_QUERY_UPDATE:
 		dbg_net("udp: UPDATE query on fd=%d not implemented\n", fd);
-		knot_ns_error_response(ns, knot_packet_id(packet),
-		                       &packet->header.flags1,
-		                       KNOT_RCODE_NOTIMPL, qbuf, resp_len);
-		res = KNOTD_EOK;
+		knot_ns_error_response_from_query(ns, packet,
+		                                  KNOT_RCODE_NOTIMPL, qbuf,
+		                                  resp_len);
+		res = KNOT_EOK;
 		break;
 		
 	/* Unhandled opcodes. */
 	case KNOT_RESPONSE_AXFR: /*!< Processed in XFR handler. */
 	case KNOT_RESPONSE_IXFR: /*!< Processed in XFR handler. */
-		knot_ns_error_response(ns, knot_packet_id(packet),
-		                       &packet->header.flags1,
-		                       KNOT_RCODE_REFUSED, qbuf,
-		                       resp_len);
-		res = KNOTD_EOK;
+		knot_ns_error_response_from_query(ns, packet,
+		                                  KNOT_RCODE_REFUSED, qbuf,
+		                                  resp_len);
+		res = KNOT_EOK;
 		break;
 			
 	/* Unknown opcodes */
 	default:
-		knot_ns_error_response(ns, knot_packet_id(packet),
-		                       &packet->header.flags1,
-		                       KNOT_RCODE_FORMERR, qbuf, resp_len);
-		res = KNOTD_EOK;
+		knot_ns_error_response_from_query(ns, packet,
+		                                  KNOT_RCODE_FORMERR, qbuf,
+		                                  resp_len);
+		res = KNOT_EOK;
 		break;
 	}
 
@@ -198,40 +197,37 @@ static inline int udp_master_recvfrom(dthread_t *thread, stat_t *thread_stat)
 	iohandler_t *h = (iohandler_t *)thread->data;
 	if (h == NULL || h->server == NULL || h->server->nameserver == NULL) {
 		dbg_net("udp: invalid parameters for udp_master_recvfrom\n");
-		return KNOTD_EINVAL;
+		return KNOT_EINVAL;
 	}
 	
 	/* Set CPU affinity to improve load distribution on multicore systems.
 	 * Partial overlapping mask to be nice to scheduler.
 	 */
-#ifdef HAVE_PTHREAD_SETAFFINITY_NP
 	int cpcount = dt_online_cpus();
 	if (cpcount > 0) {
-		unsigned tid = dt_get_id(thread);
-		cpu_set_t cpus;
-		CPU_ZERO(&cpus);
-		CPU_SET(tid % cpcount, &cpus);
-		CPU_SET((tid + 1) % cpcount, &cpus);
-		dt_setaffinity(thread, &cpus, sizeof(cpu_set_t));
+		unsigned cpu[2];
+		cpu[0] = dt_get_id(thread);
+		cpu[1] = (cpu[0] + 1) % cpcount;
+		cpu[0] = cpu[0] % cpcount;
+		dt_setaffinity(thread, cpu, 2);
 	}
-#endif
 	
 	knot_nameserver_t *ns = h->server->nameserver;
 
 	/* Initialize remote party address. */
 	sockaddr_t addr;
-	if (sockaddr_init(&addr, h->type) != KNOTD_EOK) {
+	if (sockaddr_init(&addr, h->type) != KNOT_EOK) {
 		log_server_error("Socket type %d is not supported, "
 				 "IPv6 support is probably disabled.\n",
 				 h->type);
-		return KNOTD_ENOTSUP;
+		return KNOT_ENOTSUP;
 	}
 	
 	/* Allocate buffer for answering. */
 	uint8_t *qbuf = malloc(SOCKET_MTU_SZ);
 	if (qbuf == NULL) {
 		dbg_net("udp: out of memory when allocating buffer.\n");
-		return KNOTD_ENOMEM;
+		return KNOT_ENOMEM;
 	}
 	
 	/* Duplicate socket for performance reasons on some OS's */
@@ -256,7 +252,7 @@ static inline int udp_master_recvfrom(dthread_t *thread, stat_t *thread_stat)
 		}
 
 		/* Error and interrupt handling. */
-		if (unlikely(n <= 0)) {
+		if (knot_unlikely(n <= 0)) {
 			if (errno != EINTR && errno != 0) {
 				dbg_net("udp: recvmsg() failed: %d\n",
 					  errno);
@@ -274,7 +270,7 @@ static inline int udp_master_recvfrom(dthread_t *thread, stat_t *thread_stat)
 		int rc = udp_handle(sock, qbuf, n, &resp_len, &addr, ns);
 
 		/* Send response. */
-		if (rc == KNOTD_EOK && resp_len > 0) {
+		if (rc == KNOT_EOK && resp_len > 0) {
 
 			dbg_net("udp: on fd=%d, sending answer size=%zd.\n",
 			        sock, resp_len);
@@ -298,7 +294,7 @@ static inline int udp_master_recvfrom(dthread_t *thread, stat_t *thread_stat)
 	
 	free(qbuf);
 
-	return KNOTD_EOK;
+	return KNOT_EOK;
 }
 
 #ifdef ENABLE_RECVMMSG
@@ -335,7 +331,7 @@ int udp_sendto(int sock, sockaddr_t * addrs, struct mmsghdr *msgs, size_t count)
 		}
 	}
 	
-	return KNOTD_EOK;
+	return KNOT_EOK;
 }
 
 #ifdef ENABLE_SENDMMSG
@@ -358,10 +354,10 @@ int udp_sendmmsg(int sock, sockaddr_t *_, struct mmsghdr *msgs, size_t count)
 	UNUSED(_);
 	dbg_net("udp: sending multiple responses\n");
 	if (sendmmsg(sock, msgs, count, 0) < 0) {
-		return KNOTD_ERROR;
+		return KNOT_ERROR;
 	}
 	
-	return KNOTD_EOK;
+	return KNOT_EOK;
 }
 #endif
 
@@ -374,7 +370,7 @@ static inline int udp_master_recvmmsg(dthread_t *thread, stat_t *thread_stat)
 	/* Check socket. */
 	if (sock < 0) {
 		dbg_net("udp: unable to dup() socket, finishing.\n");
-		return KNOTD_EINVAL;
+		return KNOT_EINVAL;
 	}
 
 	/* Allocate batch for N packets. */
@@ -389,7 +385,8 @@ static inline int udp_master_recvmmsg(dthread_t *thread, stat_t *thread_stat)
 		free(addrs);
 		free(iov);
 		free(msgs);
-		return KNOTD_ENOMEM;
+		close(sock);
+		return KNOT_ENOMEM;
 	}
 
 	/* Prepare batch. */
@@ -407,17 +404,14 @@ static inline int udp_master_recvmmsg(dthread_t *thread, stat_t *thread_stat)
 	/* Set CPU affinity to improve load distribution on multicore systems.
 	 * Partial overlapping mask to be nice to scheduler.
 	 */
-#ifdef HAVE_PTHREAD_SETAFFINITY_NP
 	int cpcount = dt_online_cpus();
 	if (cpcount > 0) {
-		unsigned tid = dt_get_id(thread);
-		cpu_set_t cpus;
-		CPU_ZERO(&cpus);
-		CPU_SET(tid % cpcount, &cpus);
-		CPU_SET((tid + 1) % cpcount, &cpus);
-		dt_setaffinity(thread, &cpus, sizeof(cpu_set_t));
+		unsigned cpu[2];
+		cpu[0] = dt_get_id(thread);
+		cpu[1] = (cpu[0] + 1) % cpcount;
+		cpu[0] = cpu[0] % cpcount;
+		dt_setaffinity(thread, cpu, 2);
 	}
-#endif
 
 	/* Loop until all data is read. */
 	ssize_t n = 0;
@@ -432,8 +426,10 @@ static inline int udp_master_recvmmsg(dthread_t *thread, stat_t *thread_stat)
 		}
 
 		/* Error and interrupt handling. */
-		if (unlikely(n <= 0)) {
-			if (errno != EINTR && errno != 0) {
+		if (knot_unlikely(n <= 0)) {
+			if (errno != EINTR && errno != 0 && n < 0) {
+				log_server_error("I/O failure in UDP - errno %d "
+				                 "(Linux/recvmmsg)", errno);
 				dbg_net("udp: recvmmsg() failed: %d\n",
 				        errno);
 			}
@@ -452,7 +448,7 @@ static inline int udp_master_recvmmsg(dthread_t *thread, stat_t *thread_stat)
 			size_t resp_len = msgs[i].msg_len;
 			ret = udp_handle(sock, cvec->iov_base, resp_len, &resp_len,
 			                 addrs + i, ns);
-			if (ret == KNOTD_EOK) {
+			if (ret == KNOT_EOK) {
 				msgs[i].msg_len = resp_len;
 				iov[i].iov_len = resp_len;
 			} else {
@@ -477,7 +473,7 @@ static inline int udp_master_recvmmsg(dthread_t *thread, stat_t *thread_stat)
 	free(iov);
 	free(msgs);
 	close(sock);
-	return KNOTD_EOK;
+	return KNOT_EOK;
 }
 #endif
 #endif
@@ -493,7 +489,10 @@ void __attribute__ ((constructor)) udp_master_init()
 #ifdef MSG_WAITFORONE
 	/* Check for recvmmsg() support. */
 	if (dlsym(RTLD_DEFAULT, "recvmmsg") != 0) {
-		_udp_master = udp_master_recvmmsg;
+		int r = recvmmsg(0, NULL, 0, 0, 0);
+		if (errno != ENOSYS) {
+			_udp_master = udp_master_recvmmsg;
+		}
 	}
 	
 	/* Check for sendmmsg() support. */
@@ -517,7 +516,7 @@ int udp_master(dthread_t *thread)
 	/* Check socket. */
 	if (sock < 0) {
 		dbg_net("udp: null socket recevied, finishing.\n");
-		return KNOTD_EINVAL;
+		return KNOT_EINVAL;
 	}
 
 	/* Set socket options. */
@@ -563,10 +562,10 @@ int udp_master(dthread_t *thread)
 	/* Execute proper handler. */
 	dbg_net_verb("udp: thread started (worker %p).\n", thread);
 	int ret = _udp_master(thread, thread_stat);
-	if (ret != KNOTD_EOK) {
+	if (ret != KNOT_EOK) {
 		log_server_warning("UDP answering module finished "
 		                   "with an error (%s).\n",
-		                   knotd_strerror(ret));
+		                   knot_strerror(ret));
 	}
 
 	stat_free(thread_stat);
