@@ -49,7 +49,6 @@
 #include <netdb.h>
 
 #include "libknot/libknot.h"
-#include "knot/other/error.h"
 #include "libknot/common.h"
 #include "common/skip-list.h"
 #include "common/base32hex.h"
@@ -305,12 +304,12 @@ static inline uint16_t rdata_item_size(knot_rdata_item_t item)
 	return item.raw_data[0];
 }
 
-char *rdata_dname_to_string(knot_rdata_item_t item)
+static char *rdata_dname_to_string(knot_rdata_item_t item)
 {
 	return knot_dname_to_str(item.dname);
 }
 
-char *rdata_binary_dname_to_string(knot_rdata_item_t item)
+static char *rdata_binary_dname_to_string(knot_rdata_item_t item)
 {
 	if (item.raw_data == NULL) {
 		return NULL;
@@ -334,92 +333,108 @@ char *rdata_binary_dname_to_string(knot_rdata_item_t item)
 	return str;
 }
 
-char *rdata_dns_name_to_string(knot_rdata_item_t item)
+static char *rdata_dns_name_to_string(knot_rdata_item_t item)
 {
 	return knot_dname_to_str(item.dname);
 }
 
-static char *rdata_txt_data_to_string(const uint8_t *data)
+static char *rdata_txt_data_to_string(const uint8_t *data, uint32_t *count)
 {
-	uint8_t length = data[0];
-	size_t i;
+    uint8_t length = data[0];
+    uint32_t i = 0;
+    /*
+     * * 4 because: unprintable chars look like \123.
+     * + 3 because: opening '"', closing '"' and \0 at the end.
+     * + 1 because: malloc + gcc optimization requires out_length >= 4.
+     *
+     * NOTE: length can be 0.
+     */
+    uint32_t out_length = length * 4 + 3 + 1;
 
-	if (length == 0) {
-		return NULL;
-	}
+    char *ret = malloc(out_length);
+    if (ret == NULL) {
+            ERR_ALLOC_FAILED;
+            return NULL;
+    }
+    memset(ret, 0, out_length);
 
-	/*
-	 * 3 because: opening '"', closing '"', and \0 at the end.
-	 * Times 2 because string can be all "double chars".
-	 */
-	size_t current_length = sizeof(char) * (length * 2 + 4);
-	char *ret = malloc(current_length);
-	if (ret == NULL) {
-		ERR_ALLOC_FAILED;
-		return NULL;
-	}
-	memset(ret, 0,  current_length);
+    // Opening '"'
+    strcat(ret, "\"");
 
-	strncat(ret, "\"", 3);
+    for (i = 1; i <= length; i++) {
+            char ch = (char) data[i];
+            char tmp_str[5];
 
-	for (i = 1; i <= length; i++) {
-		char ch = (char) data[i];
-		if (isprint((int)ch)) {
-			if (ch == '"' || ch == '\\') {
-				strncat(ret, "\"", 3);
-			}
-				/* for the love of god, how to this better,
-				   but w/o obscure self-made functions */
-				char tmp_str[2];
-				tmp_str[0] = ch;
-				tmp_str[1] = 0;
-				strncat(ret, tmp_str, 2);
-		} else {
-			strncat(ret, "\\", 3);
-			char tmp_str[2];
-			tmp_str[0] = ch - '0';
-			tmp_str[1] = 0;
+            if (isprint((int)ch)) {
+                    if (ch == '"' || ch == '\\') {
+                            strcat(ret, "\\");
+                    }
+                    tmp_str[0] = ch;
+                    tmp_str[1] = 0;
+                    strcat(ret, tmp_str);
+            } else {
+                    sprintf(tmp_str, "\\%03u", ch);
+                    strcat(ret, tmp_str);
+            }
+    }
 
-			strncat(ret, tmp_str, 2);
-		}
-	}
-	strncat(ret, "\"", 3);
+    // Closing '"'
+    strcat(ret, "\"");
 
-	return ret;
+    *count = length + 1; // 1 - leading length byte.
+
+    return ret;
 }
 
-char *rdata_text_to_string(knot_rdata_item_t item)
+static char *rdata_text_array_to_string(knot_rdata_item_t item)
 {
-	uint16_t size = item.raw_data[0];
-	char *ret = malloc(sizeof(char) * size * 2 + 1) ;
-	if (ret == NULL) {
-		ERR_ALLOC_FAILED;
-		return NULL;
-	}
-	memset(ret, 0, sizeof(char) * size);
-	const uint8_t *data = (uint8_t *)(item.raw_data +  1);
-	size_t read_count = 0;
-	while (read_count < size) {
-		assert(read_count <= size);
-		char *txt = rdata_txt_data_to_string(data + read_count);
-		if (txt == NULL) {
-			free(ret);
-			return NULL;
-		}
-		read_count += strlen(txt) - 1;
-		/* Create delimiter. */
-		char del[2];
-		del[0] = ' ';
-		del[1] = '\0';
-		strncat(ret, txt, strlen(txt));
-		strncat(ret, del, 2);
-		free(txt);
-	}
+    /* Create delimiter. */
+    char *del = " ";
 
-	return ret;
+    uint16_t size = item.raw_data[0];
+    /*
+     * * 6 because: item can consists of one length unprintable char strings
+     *              "\123" "\123" ...
+     * + 1 because: ending \0.
+     *
+     * NOTE: txt_size is always bigger than 4 bytes (zero string has size = 1).
+     */
+    uint32_t txt_size = size * 6 + 1;
+
+    char *ret = malloc(txt_size);
+    if (ret == NULL) {
+            ERR_ALLOC_FAILED;
+            return NULL;
+    }
+    memset(ret, 0, txt_size);
+
+    const uint8_t *data = (uint8_t *)(item.raw_data + 1);
+    uint32_t read_count = 0;
+
+    while (read_count < size) {
+            uint32_t txt_count = 0;
+
+            char *txt = rdata_txt_data_to_string(data + read_count, &txt_count);
+            if (txt == NULL) {
+                    free(ret);
+                    return NULL;
+            }
+
+            /* Append text string to output. */
+            strcat(ret, txt);
+            read_count += txt_count;
+
+            if (read_count < size) {
+                    strcat(ret, del);
+            }
+
+            free(txt);
+    }
+
+    return ret;
 }
 
-char *rdata_byte_to_string(knot_rdata_item_t item)
+static char *rdata_byte_to_string(knot_rdata_item_t item)
 {
 	assert(item.raw_data[0] == 1);
 	uint8_t data = *((uint8_t *)(item.raw_data + 1));
@@ -428,7 +443,7 @@ char *rdata_byte_to_string(knot_rdata_item_t item)
 	return ret;
 }
 
-char *rdata_short_to_string(knot_rdata_item_t item)
+static char *rdata_short_to_string(knot_rdata_item_t item)
 {
 	uint16_t data = knot_wire_read_u16(rdata_item_data(item));
 	char *ret = malloc(sizeof(char) * U16_MAX_STR_LEN);
@@ -438,7 +453,7 @@ char *rdata_short_to_string(knot_rdata_item_t item)
 	return ret;
 }
 
-char *rdata_long_to_string(knot_rdata_item_t item)
+static char *rdata_long_to_string(knot_rdata_item_t item)
 {
 	uint32_t data = knot_wire_read_u32(rdata_item_data(item));
 	char *ret = malloc(sizeof(char) * U32_MAX_STR_LEN);
@@ -447,7 +462,7 @@ char *rdata_long_to_string(knot_rdata_item_t item)
 	return ret;
 }
 
-char *rdata_a_to_string(knot_rdata_item_t item)
+static char *rdata_a_to_string(knot_rdata_item_t item)
 {
 	/* 200 seems like a little too much */
 	char *ret = malloc(sizeof(char) * 200);
@@ -458,7 +473,7 @@ char *rdata_a_to_string(knot_rdata_item_t item)
 	}
 }
 
-char *rdata_aaaa_to_string(knot_rdata_item_t item)
+static char *rdata_aaaa_to_string(knot_rdata_item_t item)
 {
 	char *ret = malloc(sizeof(char) * 200);
 	if (inet_ntop(AF_INET6, rdata_item_data(item), ret, 200)) {
@@ -468,7 +483,7 @@ char *rdata_aaaa_to_string(knot_rdata_item_t item)
 	}
 }
 
-char *rdata_rrtype_to_string(knot_rdata_item_t item)
+static char *rdata_rrtype_to_string(knot_rdata_item_t item)
 {
 	uint16_t type = knot_wire_read_u16(rdata_item_data(item));
 	const char *tmp = knot_rrtype_to_string(type);
@@ -477,7 +492,7 @@ char *rdata_rrtype_to_string(knot_rdata_item_t item)
 	return ret;
 }
 
-char *rdata_algorithm_to_string(knot_rdata_item_t item)
+static char *rdata_algorithm_to_string(knot_rdata_item_t item)
 {
 	uint8_t id = *rdata_item_data(item);
 	char *ret = malloc(sizeof(char) * MAX_RR_TYPE_LEN);
@@ -492,7 +507,7 @@ char *rdata_algorithm_to_string(knot_rdata_item_t item)
 	return ret;
 }
 
-char *rdata_certificate_type_to_string(knot_rdata_item_t item)
+static char *rdata_certificate_type_to_string(knot_rdata_item_t item)
 {
 	uint16_t id = knot_wire_read_u16(rdata_item_data(item));
 	char *ret = malloc(sizeof(char) * MAX_RR_TYPE_LEN);
@@ -507,7 +522,7 @@ char *rdata_certificate_type_to_string(knot_rdata_item_t item)
 	return ret;
 }
 
-char *rdata_period_to_string(knot_rdata_item_t item)
+static char *rdata_period_to_string(knot_rdata_item_t item)
 {
 	/* uint32 but read 16 XXX */
 	uint32_t period = knot_wire_read_u32(rdata_item_data(item));
@@ -516,7 +531,7 @@ char *rdata_period_to_string(knot_rdata_item_t item)
 	return ret;
 }
 
-char *rdata_time_to_string(knot_rdata_item_t item)
+static char *rdata_time_to_string(knot_rdata_item_t item)
 {
 	time_t time = (time_t) knot_wire_read_u32(rdata_item_data(item));
 	struct tm tm_conv;
@@ -532,7 +547,7 @@ char *rdata_time_to_string(knot_rdata_item_t item)
 	}
 }
 
-char *rdata_base32_to_string(knot_rdata_item_t item)
+static char *rdata_base32_to_string(knot_rdata_item_t item)
 {
 	int length;
 	size_t size = rdata_item_size(item);
@@ -555,7 +570,8 @@ char *rdata_base32_to_string(knot_rdata_item_t item)
 	}
 }
 
-char *rdata_base64_to_string(knot_rdata_item_t item)
+/*!< \todo Replace with function from .../common after release. */
+static char *rdata_base64_to_string(knot_rdata_item_t item)
 {
 	int length;
 	size_t size = rdata_item_size(item);
@@ -570,7 +586,7 @@ char *rdata_base64_to_string(knot_rdata_item_t item)
 	}
 }
 
-char *hex_to_string(const uint8_t *data, size_t size)
+static char *knot_hex_to_string(const uint8_t *data, size_t size)
 {
 	static const char hexdigits[] = {
 		'0', '1', '2', '3', '4', '5', '6', '7',
@@ -593,7 +609,7 @@ char *hex_to_string(const uint8_t *data, size_t size)
 
 char *rdata_hex_to_string(knot_rdata_item_t item)
 {
-	return hex_to_string(rdata_item_data(item), rdata_item_size(item));
+	return knot_hex_to_string(rdata_item_data(item), rdata_item_size(item));
 }
 
 char *rdata_hexlen_to_string(knot_rdata_item_t item)
@@ -605,8 +621,8 @@ char *rdata_hexlen_to_string(knot_rdata_item_t item)
 		ret[1] = '\0';
 		return ret;
 	} else {
-		return hex_to_string(rdata_item_data(item) + 1,
-		                     rdata_item_size(item) - 1);
+		return knot_hex_to_string(rdata_item_data(item) + 1,
+		                          rdata_item_size(item) - 1);
 	}
 }
 
@@ -622,8 +638,8 @@ char *rdata_nsap_to_string(knot_rdata_item_t item)
 
 	/* String is already terminated. */
 	memcpy(ret, "0x", strlen("0x"));
-	char *converted = hex_to_string(rdata_item_data(item),
-	                                rdata_item_size(item));
+	char *converted = knot_hex_to_string(rdata_item_data(item),
+	                                     rdata_item_size(item));
 	if (converted == NULL) {
 		return NULL;
 	}
@@ -707,9 +723,7 @@ char *rdata_services_to_string(knot_rdata_item_t item)
 	if (proto) {
 		int i;
 
-		/*!< \todo #1863 see below, but we can trust getprotobynumber... */
 		strncpy(ret, proto->p_name, strlen(proto->p_name));
-
 		strncat(ret, " ", 2);
 
 		for (i = 0; i < bitmap_size * 8; ++i) {
@@ -718,12 +732,6 @@ char *rdata_services_to_string(knot_rdata_item_t item)
 					getservbyport((int)htons(i),
 						      proto->p_name);
 				if (service) {
-					/*!< \todo #1863 
-					 * using strncat with strlen
-					 * does not make a whole lot of sense.
-					 * At least it will crash wil
-					 * Use max length of service name!
-					 */
 					strncat(ret, service->s_name,
 					        strlen(service->s_name));
 					strncat(ret, " ", 2);
@@ -738,37 +746,6 @@ char *rdata_services_to_string(knot_rdata_item_t item)
 	}
 
 	return ret;
-
-	/*
-	int result = 0;
-	uint8_t protocol_number = buffer_read_u8(&packet);
-	ssize_t bitmap_size = buffer_remaining(&packet);
-	uint8_t *bitmap = buffer_current(&packet);
-	struct protoent *proto = getprotobynumber(protocol_number);
-
-
-	if (proto) {
-		int i;
-
-		strcpy(ret, proto->p_name);
-
-		for (i = 0; i < bitmap_size * 8; ++i) {
-			if (get_bit(bitmap, i)) {
-				struct servent *service =
-					getservbyport((int)htons(i),
-						      proto->p_name);
-				if (service) {
-					buffer_printf(output, " %s",
-						      service->s_name);
-				} else {
-					buffer_printf(output, " %d", i);
-				}
-			}
-		}
-		result = 1;
-	}
-	return ret;
-	*/
 }
 
 char *rdata_ipsecgateway_to_string(knot_rdata_item_t item,
@@ -830,17 +807,15 @@ char *rdata_nxt_to_string(knot_rdata_item_t item)
 
 char *rdata_nsec_to_string(knot_rdata_item_t item)
 {
-	/* CLEANUP */
-//	int insert_space = 0;
-
 	char *ret = malloc(sizeof(char) * MAX_NSEC_BIT_STR_LEN);
-
+	if (ret == NULL) {
+		ERR_ALLOC_FAILED;
+		return NULL;
+	}
 	memset(ret, 0, MAX_NSEC_BIT_STR_LEN);
-
 	uint8_t *data = rdata_item_data(item);
 
 	int increment = 0;
-
 	for (int i = 0; i < rdata_item_size(item); i += increment) {
 		increment = 0;
 		uint8_t window = data[i];
@@ -870,33 +845,6 @@ char *rdata_nsec_to_string(knot_rdata_item_t item)
 	}
 
 	return ret;
-
-	/* CLEANUP */
-/*	while (buffer_available(&packet, 2)) {
-		uint8_t window = buffer_read_u8(&packet);
-		uint8_t bitmap_size = buffer_read_u8(&packet);
-		uint8_t *bitmap = buffer_current(&packet);
-		int i;
-
-		if (!buffer_available(&packet, bitmap_size)) {
-			buffer_set_position(output, saved_position);
-			return 0;
-		}
-
-		for (i = 0; i < bitmap_size * 8; ++i) {
-			if (get_bit(bitmap, i)) {
-				buffer_printf(output,
-					      "%s%s",
-					      insert_space ? " " : "",
-					      rrtype_to_string(
-						      window * 256 + i));
-				insert_space = 1;
-			}
-		}
-		buffer_skip(&packet, bitmap_size);
-	}
-
-	return 1; */
 }
 
 char *rdata_unknown_to_string(knot_rdata_item_t item)
@@ -912,7 +860,7 @@ char *rdata_unknown_to_string(knot_rdata_item_t item)
 	snprintf(ret + strlen("\\# "),
 	         strlen("\\# ") + U16_MAX_STR_LEN + 1, "%lu ",
 		 (unsigned long) size);
-	char *converted = hex_to_string(rdata_item_data(item), size);
+	char *converted = knot_hex_to_string(rdata_item_data(item), size);
 	strncat(ret, converted, size * 2 + 1);
 	free(converted);
 	return ret;
@@ -928,7 +876,8 @@ typedef char * (*item_to_string_t)(knot_rdata_item_t);
 static item_to_string_t item_to_string_table[KNOT_RDATA_ZF_UNKNOWN + 1] = {
 	rdata_dname_to_string,
 	rdata_dns_name_to_string,
-	rdata_text_to_string,
+	rdata_text_array_to_string,
+	rdata_text_array_to_string,
 	rdata_byte_to_string,
 	rdata_short_to_string,
 	rdata_long_to_string,
@@ -959,14 +908,13 @@ char *rdata_item_to_string(knot_rdata_zoneformat_t type,
 	return item_to_string_table[type](item);
 }
 
-/* CLEANUP */
-/*void knot_zone_tree_apply_inorder(knot_zone_t *zone,
-                              void (*function)(knot_node_t *node, void *data),
-                              void *data); */
-
 int rdata_dump_text(const knot_rdata_t *rdata, uint16_t type, FILE *f,
                      const knot_rrset_t *rrset)
 {
+	if (rdata == NULL || rrset == NULL) {
+		return KNOT_EINVAL;
+	}
+	
 	knot_rrtype_descriptor_t *desc =
 		knot_rrtype_descriptor_by_type(type);
 	char *item_str = NULL;
@@ -988,7 +936,7 @@ int rdata_dump_text(const knot_rdata_t *rdata, uint16_t type, FILE *f,
 
 		if (item_str == NULL) {
 			/* Fatal error. */
-			return KNOTD_ERROR;
+			return KNOT_ERROR;
 		}
 
 		if (i != rdata->count - 1) {
@@ -1001,7 +949,7 @@ int rdata_dump_text(const knot_rdata_t *rdata, uint16_t type, FILE *f,
 	}
 	fprintf(f, "\n");
 
-	return KNOTD_EOK;
+	return KNOT_EOK;
 }
 
 void dump_rrset_header(const knot_rrset_t *rrset, FILE *f)
@@ -1021,8 +969,8 @@ int rrsig_set_dump_text(knot_rrset_t *rrsig, FILE *f)
 
 	while (tmp->next != rrsig->rdata) {
 		int ret = rdata_dump_text(tmp, KNOT_RRTYPE_RRSIG, f, rrsig);
-		if (ret != KNOTD_EOK) {
-			return KNOTD_ERROR;
+		if (ret != KNOT_EOK) {
+			return KNOT_ERROR;
 		}
 
 		dump_rrset_header(rrsig, f);
@@ -1030,35 +978,39 @@ int rrsig_set_dump_text(knot_rrset_t *rrsig, FILE *f)
 	}
 
 	int ret = rdata_dump_text(tmp, KNOT_RRTYPE_RRSIG, f, rrsig);
-	if (ret != KNOTD_EOK) {
-		return KNOTD_ERROR;
+	if (ret != KNOT_EOK) {
+		return KNOT_ERROR;
 	}
 
-	return KNOTD_EOK;
+	return KNOT_EOK;
 }
 
 
 int rrset_dump_text(const knot_rrset_t *rrset, FILE *f)
 {
-	dump_rrset_header(rrset, f);
-	knot_rdata_t *tmp = rrset->rdata;
-
-	while (tmp->next != rrset->rdata) {
-		int ret = rdata_dump_text(tmp, rrset->type, f, rrset);
-		if (ret != KNOTD_EOK) {
-			return ret;
-		}
+	if (rrset->rdata != NULL) { // No sense in dumping empty RR
 		dump_rrset_header(rrset, f);
-		tmp = tmp->next;
+
+		knot_rdata_t *tmp = rrset->rdata;
+
+		while (tmp->next != rrset->rdata) {
+			int ret = rdata_dump_text(tmp, rrset->type, f, rrset);
+			if (ret != KNOT_EOK) {
+				return ret;
+			}
+			dump_rrset_header(rrset, f);
+			tmp = tmp->next;
+		}
+
+		rdata_dump_text(tmp, rrset->type, f, rrset);
 	}
 
-	rdata_dump_text(tmp, rrset->type, f, rrset);
 	knot_rrset_t *rrsig_set = rrset->rrsigs;
 	if (rrsig_set != NULL) {
 		rrsig_set_dump_text(rrsig_set, f);
 	}
 
-	return KNOTD_EOK;
+	return KNOT_EOK;
 }
 
 struct dump_param {
@@ -1075,7 +1027,7 @@ int apex_node_dump_text(knot_node_t *node, FILE *f)
 		                                &dummy_rrset);
 	assert(tmp_rrset);
 	int ret = rrset_dump_text(tmp_rrset, f);
-	if (ret != KNOTD_EOK) {
+	if (ret != KNOT_EOK) {
 		return ret;
 	}
 
@@ -1085,7 +1037,7 @@ int apex_node_dump_text(knot_node_t *node, FILE *f)
 	for (int i = 0; i < node->rrset_count; i++) {
 		if (rrsets[i]->type != KNOT_RRTYPE_SOA) {
 			ret = rrset_dump_text(rrsets[i], f);
-			if (ret != KNOTD_EOK) {
+			if (ret != KNOT_EOK) {
 				return ret;
 			}
 		}
@@ -1093,7 +1045,7 @@ int apex_node_dump_text(knot_node_t *node, FILE *f)
 
 	free(rrsets);
 
-	return KNOTD_EOK;
+	return KNOT_EOK;
 }
 
 void node_dump_text(knot_node_t *node, void *data)
@@ -1122,7 +1074,7 @@ void node_dump_text(knot_node_t *node, void *data)
 int zone_dump_text(knot_zone_contents_t *zone, FILE *f)
 {
 	if (f == NULL) {
-		return KNOT_EBADARG;
+		return KNOT_EINVAL;
 	}
 
 	fprintf(f, ";Dumped using %s v. %s\n", PACKAGE_NAME, PACKAGE_VERSION);
