@@ -29,7 +29,6 @@
 #include "common/crc.h"
 #include "knot/other/debug.h"
 #include "common/skip-list.h"
-#include "libknot/util/error.h"
 #include "semantic-check.h"
 
 #define ZONECHECKS_VERBOSE
@@ -58,7 +57,7 @@ static inline int write_to_file_crc(const void *src,
                                      crc_t *crc)
 {
 	if (src == NULL || fd < 0) {
-		return KNOT_EBADARG;
+		return KNOT_EINVAL;
 	}
 	ssize_t rc = write(fd, src, size * n);
 	if (rc != size * n) {
@@ -82,7 +81,7 @@ static inline int write_to_stream(const void *src,
                                   size_t *written_bytes)
 {
 	if (src == NULL || stream == NULL || written_bytes == NULL) {
-		return KNOT_EBADARG;
+		return KNOT_EINVAL;
 	}
 	
 	/* Check that the stream boundary will not be crossed. */
@@ -108,7 +107,7 @@ static int write_wrapper(const void *src,
 {
 	if (src == NULL) {
 		dbg_zdump("zdump: write_wrapper: NULL source.\n");
-		return KNOT_EBADARG;
+		return KNOT_EINVAL;
 	}
 	
 	dbg_zdump_detail("zdump: write_wrapper: Writing %d bytes to fd: %d.\n",
@@ -233,7 +232,7 @@ static int knot_labels_dump_binary(const knot_dname_t *dname, int fd,
 {
 	if (dname == NULL) {
 		dbg_zdump("zdump: dump_labels: NULL dname.\n");
-		return KNOT_EBADARG;
+		return KNOT_EINVAL;
 	}
 	
 	uint16_t label_count = dname->label_count;
@@ -266,7 +265,7 @@ static int knot_dname_dump_binary(const knot_dname_t *dname, int fd,
 {
 	if (dname == NULL) {
 		dbg_zdump("zdump: dump_dname: NULL dname.\n");
-		return KNOT_EBADARG;
+		return KNOT_EINVAL;
 	}
 	
 	/*! \todo too big */
@@ -295,7 +294,7 @@ static int dump_dname_with_id(const knot_dname_t *dname, int fd,
 {
 	if (dname == NULL) {
 		dbg_zdump("zdump: dump_dname: NULL dname.\n");
-		return KNOT_EBADARG;
+		return KNOT_EINVAL;
 	}
 	
 	uint32_t id = dname->id;
@@ -323,7 +322,7 @@ static int knot_rdata_dump_binary(knot_rdata_t *rdata,
 {
 	if (rdata == NULL) {
 		dbg_zdump("zdump: dump_rdata: NULL rdata.\n");
-		return KNOT_EBADARG;
+		return KNOT_EINVAL;
 	}
 	knot_rrtype_descriptor_t *desc =
 		knot_rrtype_descriptor_by_type(type);
@@ -360,6 +359,11 @@ static int knot_rdata_dump_binary(knot_rdata_t *rdata,
 				rdata->items[i].dname) {
 				wildcard = rdata->items[i].dname->node->owner;
 			}
+			
+			dbg_zdump_detail("zdump: dump_rdata: "
+			                 "Writing dname: %s.\n",
+			                 knot_dname_to_str(
+						rdata->items[i].dname));
 
 			if (use_ids) {
 				/* Write ID. */
@@ -392,6 +396,8 @@ static int knot_rdata_dump_binary(knot_rdata_t *rdata,
 			/*! \todo Does not have to be so complex.
 			 *        Create extra variable. */
 			if (rdata->items[i].dname->node != NULL && !wildcard) {
+				dbg_zdump("zdump: dump_rdata: "
+				          "This dname is in the zone.\n");
 				if (!write_wrapper((uint8_t *)"\1",
 				                   sizeof(uint8_t), 1, fd,
 				                   stream, max_size,
@@ -401,6 +407,8 @@ static int knot_rdata_dump_binary(knot_rdata_t *rdata,
 					return KNOT_ERROR;
 				}
 			} else {
+				dbg_zdump("zdump: dump_rdata: "
+				          "This dname is not in the zone.\n");
 				if (!write_wrapper((uint8_t *)"\0",
 				                   sizeof(uint8_t),
 				                   1, fd,
@@ -423,6 +431,9 @@ static int knot_rdata_dump_binary(knot_rdata_t *rdata,
 				}
 				
 				uint32_t wildcard_id = wildcard->id;
+				dbg_zdump("zdump: dump_rdata: "
+				          "This dname is covered by wc (%s).\n",
+				          knot_dname_to_str(wildcard));
 				if (!write_wrapper(&wildcard_id,
 				                   sizeof(wildcard_id), 1,
 				                   fd, stream, max_size,
@@ -482,7 +493,7 @@ static int knot_rrsig_set_dump_binary(knot_rrset_t *rrsig, int fd,
 {
 	if (rrsig == NULL) {
 		dbg_zdump("zdump: dump_rrsig: NULL RRSIG.\n");
-		return KNOT_EBADARG;
+		return KNOT_EINVAL;
 	}
 	
 dbg_zdump_exec_detail(
@@ -558,7 +569,7 @@ static int knot_rrset_dump_binary(const knot_rrset_t *rrset, int fd,
 {
 	if (rrset == NULL) {
 		dbg_zdump("zdump: dump_rrset: NULL RRSet.\n");
-		return KNOT_EBADARG;
+		return KNOT_EINVAL;
 	}
 	
 	dbg_zdump_exec_detail(
@@ -567,7 +578,11 @@ static int knot_rrset_dump_binary(const knot_rrset_t *rrset, int fd,
 	                         "Dumping RRSet \\w owner: %s.\n",
 		                 name);
 		free(name);
-	);	
+	);
+		
+	if (!use_ids) {
+		assert(rrset->rrsigs == NULL);
+	}
 	
 	if (!use_ids) {
 		/*!< \todo IDs in changeset do no good. Change loading too. */
@@ -602,10 +617,14 @@ static int knot_rrset_dump_binary(const knot_rrset_t *rrset, int fd,
 
 	/* Calculate rrset rdata count. */
 	knot_rdata_t *tmp_rdata = rrset->rdata;
-	while(tmp_rdata->next != rrset->rdata) {
+	while(tmp_rdata && (tmp_rdata->next != rrset->rdata)) {
 		tmp_rdata = tmp_rdata->next;
 		rdata_count++;
 	}
+	
+	if (rrset->rdata == NULL) {
+		rdata_count = 0;
+	}
 
 	if (!write_wrapper(&rdata_count, sizeof(rdata_count), 1, fd,
 	                   stream, max_size, written_bytes, crc)) {
@@ -619,29 +638,35 @@ static int knot_rrset_dump_binary(const knot_rrset_t *rrset, int fd,
 	}
 	
 	dbg_zdump_verb("zdump: rrset_dump_binary: Static data dumped.\n");
+	
+	if (rdata_count != 0) {
+	
+		tmp_rdata = rrset->rdata;
 
-	tmp_rdata = rrset->rdata;
-
-	while (tmp_rdata->next != rrset->rdata) {
+		while (tmp_rdata->next != rrset->rdata) {
+			int ret = knot_rdata_dump_binary(tmp_rdata, rrset->type,
+			                                 fd, use_ids,
+			                                 stream, max_size,
+			                                 written_bytes, crc);
+			if (ret != KNOT_EOK) {
+				dbg_zdump("zdump: rrset_to_binary: Could not "
+				          "dump "
+				          "rdata. Reason: %s.\n",
+				          knot_strerror(ret));
+				return ret;
+			}
+			tmp_rdata = tmp_rdata->next;
+		}
+	
 		int ret = knot_rdata_dump_binary(tmp_rdata, rrset->type,
 		                                 fd, use_ids,
-		                                 stream, max_size,
-		                                 written_bytes, crc);
+		                                 stream,
+		                                 max_size, written_bytes, crc);
 		if (ret != KNOT_EOK) {
 			dbg_zdump("zdump: rrset_to_binary: Could not dump "
 			          "rdata. Reason: %s.\n", knot_strerror(ret));
 			return ret;
 		}
-		tmp_rdata = tmp_rdata->next;
-	}
-	
-	int ret = knot_rdata_dump_binary(tmp_rdata, rrset->type, fd, use_ids,
-	                                 stream,
-	                                 max_size, written_bytes, crc);
-	if (ret != KNOT_EOK) {
-		dbg_zdump("zdump: rrset_to_binary: Could not dump "
-		          "rdata. Reason: %s.\n", knot_strerror(ret));
-		return ret;
 	}
 	
 	dbg_zdump_verb("zdump: rrset_dump_binary: Rdata dumped.\n");
@@ -672,7 +697,7 @@ static int knot_node_dump_binary(knot_node_t *node, int fd,
 {
 	if (node == NULL) {
 		dbg_zdump("zdump: dump_node: NULL node.\n");
-		return KNOT_EBADARG;
+		return KNOT_EINVAL;
 	}
 	
 	/* first write dname */
@@ -801,7 +826,7 @@ static void dump_dname_from_tree(knot_dname_t *dname,
 		fd = *fd_pointer;
 	} else {
 		dbg_zdump("zdump: dump_dname_from_tree: Bad fd.\n");
-		arg->error_code = KNOT_EBADARG;
+		arg->error_code = KNOT_EINVAL;
 		return;
 	}
 	
@@ -894,9 +919,10 @@ int knot_zdump_binary(knot_zone_contents_t *zone, int fd,
                       int do_checks, const char *sfilename,
                       crc_t *crc)
 {
+
 	if (fd < 0 || sfilename == NULL) {
 		dbg_zdump("zdump: Bad arguments.\n");
-		return KNOT_EBADARG;
+		return KNOT_EINVAL;
 	}
 	
 	dbg_zdump("zdump: Dumping zone %p.\n", zone);
@@ -1074,7 +1100,7 @@ int knot_zdump_rrset_serialize(const knot_rrset_t *rrset, uint8_t *stream,
 	if (stream == NULL || rrset == NULL ||
 	    written_bytes == NULL) {
 		dbg_zdump("zdump: rrset_serialize: Bad arguments.\n");
-		return KNOT_EBADARG;
+		return KNOT_EINVAL;
 	}
 	
 	*written_bytes = 0;
