@@ -30,7 +30,6 @@
 #include "util/debug.h"
 #include "hash/cuckoo-hash-table.h"
 #include "hash/hash-functions.h"
-#include "common/dynamic-array.h"
 
 /*----------------------------------------------------------------------------*/
 /* Macros and inline functions                                                */
@@ -283,6 +282,8 @@ static int ck_stash_is_full(const ck_hash_table_t *table)
  */
 static inline void ck_clear_item(ck_hash_table_item_t **item)
 {
+	dbg_stash("[EMPTY STASH] [CREATE] setting item %p (%p) to NULL.\n",
+	          item, *item);
 	*item = NULL;
 }
 
@@ -320,51 +321,13 @@ static inline void ck_swap_items(ck_hash_table_item_t **item1,
 static inline void ck_put_item(ck_hash_table_item_t **to,
                                ck_hash_table_item_t *item)
 {
+	if (item == NULL) {
+		dbg_stash("[EMPTY STASH] [CREATE] Putting NULL to item %p.\n",
+		          to);
+	}
 	*to = item;
 }
 
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief Checks if the hash was already used twice.
- *
- * If yes, it means we entered a loop in the hashing process, so we must stop.
- * Otherwise it remembers that we used the hash.
- *
- * \note According to Kirsch, et al. a check that at most one hash was used
- *       twice should be sufficient. We will retain our version for now.
- *
- * \param used Array of used table indices (hashes).
- * \param hash Hash to check.
- *
- * \retval -1 if the hash was already used twice.
- * \retval -2 if an error occured.
- * \retval 0 if the hash was not used twice yet.
- */
-static uint ck_check_used_twice(da_array_t *used, uint32_t hash)
-{
-	uint i = 0, found = 0;
-	while (i < da_get_count(used) && found < 2) {
-		if (((uint *)(da_get_items(used)))[i] == hash) {
-			++found;
-		}
-		++i;
-	}
-
-	if (found == 2) {
-		dbg_ck_hash("Hashing entered infinite loop.\n");
-		return -1;
-	} else {
-		if (da_reserve(used, 1) < 0) {
-			ERR_ALLOC_FAILED;
-			return -2;
-		}
-		((uint *)da_get_items(used))[da_get_count(used)] = hash;
-		da_occupy(used, 1);
-		assert(da_get_count(used) < RELOCATIONS_MAX);
-		return 0;
-	}
-}
-
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Compares the key of item with the given key.
@@ -419,12 +382,14 @@ static ck_hash_table_item_t **ck_find_in_stash(const ck_hash_table_t *table,
 		 *        non-NULL.
 		 */
 		if (item->item && ck_items_match(item->item, key, length)) {
-			dbg_ck("Comparing item in stash (key: %.*s (size %zu))"
-			         "with searched item (key %.*s (size %u)).\n",
-			         (int)item->item->key_length, item->item->key,
-			         item->item->key_length, (int)length, key,
-			         length);
+			dbg_ck_detail("Comparing item in stash (key: %.*s (size"
+			    " %zu)) with searched item (key %.*s (size %u)).\n",
+			    (int)item->item->key_length, item->item->key,
+			    item->item->key_length, (int)length, key, length);
 			return &item->item;
+		} else if (item->item == NULL) {
+			dbg_stash("[EMPTY STASH] [FIND] STASH ITEM IS EMPTY: "
+			          "%p (%p)\n", item, item->item);
 		}
 		item = item->next;
 	}
@@ -448,18 +413,18 @@ static ck_hash_table_item_t **ck_find_gen(const ck_hash_table_t *table,
                                           size_t length, uint8_t generation)
 {
 	uint32_t hash;
-	dbg_ck("Finding item in generation: %u\n", generation);
+	dbg_ck_verb("Finding item in generation: %u\n", generation);
 
 	// check hash tables
 	for (uint t = 0; t < table->table_count; ++t) {
 		hash = HASH(&table->hash_system, key, length,
 		            table->table_size_exp, generation, t);
 
-		dbg_ck("Hash: %u, key: %.*s\n", hash, (int)length, key);
-		dbg_ck("Table %d, hash: %u, item: %p\n", t + 1, hash,
-		         table->tables[t][hash]);
+		dbg_ck_detail("Hash: %u, key: %.*s\n", hash, (int)length, key);
+		dbg_ck_detail("Table %d, hash: %u, item: %p\n", t + 1, hash,
+		              table->tables[t][hash]);
 		if (table->tables[t][hash] != NULL) {
-			dbg_ck("Table %u, key: %.*s, value: %p, key "
+			dbg_ck_detail("Table %u, key: %.*s, value: %p, key "
 			         "length: %zu\n",
 			         t + 1, (int)table->tables[t][hash]->key_length,
 			         table->tables[t][hash]->key,
@@ -475,16 +440,16 @@ static ck_hash_table_item_t **ck_find_gen(const ck_hash_table_t *table,
 	}
 
 	// try to find in stash
-	dbg_ck("Searching in stash...\n");
+	dbg_ck_verb("Searching in stash...\n");
 
 	ck_hash_table_item_t **found =
 	        ck_find_in_stash(table, key, length);
 
-	dbg_ck("Found pointer: %p\n", found);
+	dbg_ck_verb("Found pointer: %p\n", found);
 	if (found != NULL) {
-		dbg_ck("Stash, key: %.*s, value: %p, key length: %zu\n",
-		         (int)(*found)->key_length, (*found)->key,
-	                 (*found)->value, (*found)->key_length);
+		dbg_ck_verb("Stash, key: %.*s, value: %p, key length: %zu\n",
+		            (int)(*found)->key_length, (*found)->key,
+		            (*found)->value, (*found)->key_length);
 	}
 
 	// ck_find_in_buffer returns NULL if not found, otherwise pointer to
@@ -519,6 +484,136 @@ static ck_hash_table_item_t **ck_find_item_nc(const ck_hash_table_t *table,
 	return found;
 }
 
+/*----------------------------------------------------------------------------*/
+/* Lightweight dynamic array for keeping track of used items.                 */
+/*----------------------------------------------------------------------------*/
+
+typedef struct ck_used {
+	uint32_t *items;
+	uint array_count;
+	size_t *counts;
+	size_t allocated;
+} ck_used_t;
+
+/*----------------------------------------------------------------------------*/
+
+static int ck_used_create(ck_used_t *used, uint table_count)
+{
+	// all tables in one array
+	used->items = malloc(table_count * RELOCATIONS_DEFAULT
+	                     * sizeof(uint32_t));
+	if (used->items == NULL) {
+		return -1;
+	}
+
+	used->counts = malloc(table_count * sizeof(size_t));
+	if (used->counts == NULL) {
+		free(used->items);
+		return -1;
+	}
+
+	used->array_count = table_count;
+	used->allocated = RELOCATIONS_DEFAULT;
+
+	for (int i = 0; i < table_count; ++i) {
+		used->counts[i] = 0;
+	}
+
+	memset(used->items, 0,
+	       used->array_count * used->allocated * sizeof(uint32_t));
+
+	return 0;
+}
+
+/*----------------------------------------------------------------------------*/
+
+static void ck_used_free(ck_used_t *used)
+{
+	free(used->items);
+	free(used->counts);
+	used->allocated = 0;
+}
+
+/*----------------------------------------------------------------------------*/
+
+static int ck_used_add(ck_used_t *used, uint table_nr, uint32_t to_add)
+{
+	dbg_ck_hash_verb("1) Table nr: %u, count: %zu, allocated: %zu\n",
+	                 table_nr, used->counts[table_nr], used->allocated);
+
+	if (used->counts[table_nr] == used->allocated) {
+		dbg_ck_hash_verb("Reallocating...\n");
+		size_t allocated_new = used->allocated * 2;
+		uint32_t *items_new = malloc(used->array_count * allocated_new
+		                             * sizeof(uint32_t));
+		if (items_new == NULL) {
+			return -1;
+		}
+
+		memcpy(items_new, used->items,
+		       used->allocated * used->array_count);
+
+		uint32_t *old_items = used->items;
+
+		used->items = items_new;
+		used->allocated = allocated_new;
+
+		free(old_items);
+	}
+
+	dbg_ck_hash_verb("2) Table nr: %u, count: %zu, allocated: %zu\n",
+	                 table_nr, used->counts[table_nr], used->allocated);
+
+	assert(used->counts[table_nr] < used->allocated);
+	used->items[table_nr * used->allocated + used->counts[table_nr]]
+	            = to_add;
+	++used->counts[table_nr];
+
+	dbg_ck_hash_verb("3)Table nr: %u, count: %zu, allocated: %zu\n",
+	                 table_nr, used->counts[table_nr], used->allocated);
+
+	return 0;
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Checks if the hash was already used twice.
+ *
+ * If yes, it means we entered a loop in the hashing process, so we must stop.
+ * Otherwise it remembers that we used the hash.
+ *
+ * \note According to Kirsch, et al. a check that at most one hash was used
+ *       twice should be sufficient. We will retain our version for now.
+ *
+ * \param used Array of used table indices (hashes).
+ * \param hash Hash to check.
+ *
+ * \retval -1 if the hash was already used twice.
+ * \retval -2 if an error occured.
+ * \retval 0 if the hash was not used twice yet.
+ */
+static int ck_check_used_twice(ck_used_t *used, uint table_nr,
+                               uint32_t hash)
+{
+	uint i = 0, found = 0;
+
+	while (i < used->counts[table_nr] && found < 2) {
+		if (used->items[table_nr * used->allocated + i] == hash) {
+			++found;
+		}
+		++i;
+	}
+
+	if (found == 2) {
+		dbg_ck_hash("Hashing entered infinite loop.\n");
+		return -1;
+	} else {
+		return ck_used_add(used, table_nr, hash);
+	}
+}
+
+/*----------------------------------------------------------------------------*/
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Hashes the given item using the given generation.
@@ -532,20 +627,26 @@ static ck_hash_table_item_t **ck_find_item_nc(const ck_hash_table_t *table,
  *
  * \retval 0 if successful and no loop occured.
  * \retval 1 if a loop occured and the item was inserted to the \a free place.
+ * \retval < 0 if an error occured.
  */
 static int ck_hash_item(ck_hash_table_t *table, ck_hash_table_item_t **to_hash,
                         ck_hash_table_item_t **free, uint8_t generation)
 {
-	da_array_t used[table->table_count];
-	for (uint i = 0; i < table->table_count; ++i) {
-		da_initialize(&used[i], RELOCATIONS_DEFAULT, sizeof(uint));
+//	da_array_t used[table->table_count];
+//	for (uint i = 0; i < table->table_count; ++i) {
+//		da_initialize(&used[i], RELOCATIONS_DEFAULT, sizeof(uint));
+//	}
+	ck_used_t used;
+	int ret = ck_used_create(&used, table->table_count);
+	if (ret != 0) {
+		return -1;
 	}
 
 	// hash until empty cell is encountered or until loop appears
 
-	dbg_ck_hash("Hashing key: %.*s of size %zu.\n",
-	              (int)(*to_hash)->key_length, (*to_hash)->key,
-	              (*to_hash)->key_length);
+	dbg_ck_hash_verb("Hashing key: %.*s of size %zu.\n",
+	                 (int)(*to_hash)->key_length, (*to_hash)->key,
+	                 (*to_hash)->key_length);
 
 	uint next_table = 0;
 
@@ -553,21 +654,24 @@ static int ck_hash_item(ck_hash_table_t *table, ck_hash_table_item_t **to_hash,
 	                     (*to_hash)->key_length, table->table_size_exp,
 	                     generation, next_table);
 
-	dbg_ck_hash("New hash: %u.\n", hash);
+	dbg_ck_hash_detail("New hash: %u.\n", hash);
 	assert(hash < hashsize(table->table_size_exp));
 
-	((uint *)da_get_items(&used[next_table]))
-	[da_get_count(&used[next_table])] = hash;
+	ret = ck_used_add(&used, next_table, hash);
+	if (ret != 0) {
+		return -2;
+	}
+
 	ck_hash_table_item_t **next = &table->tables[next_table][hash];
-	dbg_ck_hash("Item to be moved: %p, place in table: %p\n",
-	              *next, next);
+	dbg_ck_hash_detail("Item to be moved: %p, place in table: %p\n",
+	                   *next, next);
 	ck_hash_table_item_t **moving = to_hash;
 
 	int loop = 0;
 
 	while (*next != NULL) {
-		dbg_ck_hash("Swapping items to hash: %p and Moving: %p\n",
-		              to_hash, moving);
+		dbg_ck_hash_detail("Swapping items: To hash: %p, Moving: %p\n",
+		                   to_hash, moving);
 		ck_swap_items(to_hash, moving); // first time it's unnecessary
 
 		// set the generation of the inserted item to the next
@@ -575,9 +679,10 @@ static int ck_hash_item(ck_hash_table_t *table, ck_hash_table_item_t **to_hash,
 
 		moving = next;
 
-		dbg_ck_hash("Moving item from table %u, key: %.*s, hash %u ",
-		              next_table + 1, (int)(*moving)->key_length,
-		              (*moving)->key, hash);
+		dbg_ck_hash_detail("Moving item from table %u, key: %.*s, hash "
+		                   "%u \n", next_table + 1,
+		                   (int)(*moving)->key_length,
+		                   (*moving)->key, hash);
 
 		// if rehashing and the 'next' item is from the old generation,
 		// start from table 1
@@ -594,31 +699,31 @@ static int ck_hash_item(ck_hash_table_t *table, ck_hash_table_item_t **to_hash,
 
 		next = &table->tables[next_table][hash];
 
-		dbg_ck_hash("to table %u, hash %u, item: %p, place: %p\n",
-		              next_table + 1, hash, *next, next);
+		dbg_ck_hash_detail("To table %u, hash %u, item: %p, place: %p"
+		                   "\n", next_table + 1, hash, *next, next);
 
 		if ((*next) != NULL) {
-			dbg_ck_hash("Table %u, hash: %u, key: %.*s\n",
-			              next_table + 1, hash,
-			              (int)(*next)->key_length, (*next)->key);
+			dbg_ck_hash_detail("Table %u, hash: %u, key: %.*s\n",
+			                next_table + 1, hash,
+			                (int)(*next)->key_length, (*next)->key);
 		}
 
 		// check if this cell wasn't already used in this item's hashing
-		if (ck_check_used_twice(&used[next_table], hash) != 0) {
+		if (ck_check_used_twice(&used, next_table, hash) != 0) {
 			next = free;
 			loop = -1;
 			break;
 		}
 	}
 
-	dbg_ck_hash("Putting pointer %p (*moving) to item %p (next).\n",
-	              *moving, next);
+	dbg_ck_hash_detail("Putting pointer %p (*moving) to item %p (next).\n",
+	                   *moving, next);
 
 	ck_put_item(next, *moving);
 	// set the new generation for the inserted item
 	SET_GENERATION(&(*next)->timestamp, generation);
-	dbg_ck_hash("Putting pointer %p (*old) to item %p (moving).\n",
-	              *to_hash, moving);
+	dbg_ck_hash_detail("Putting pointer %p (*old) to item %p (moving).\n",
+	                   *to_hash, moving);
 
 	ck_put_item(moving, *to_hash);
 
@@ -626,9 +731,7 @@ static int ck_hash_item(ck_hash_table_t *table, ck_hash_table_item_t **to_hash,
 	SET_GENERATION(&(*moving)->timestamp, generation);
 	*to_hash = NULL;
 
-	for (uint i = 0; i < table->table_count; ++i) {
-		da_destroy(&used[i]);
-	}
+	ck_used_free(&used);
 
 	return loop;
 }
@@ -670,6 +773,10 @@ static void ck_rollback_rehash(ck_hash_table_t *table)
  */
 int ck_add_to_stash(ck_hash_table_t *table, ck_hash_table_item_t *item)
 {
+	if (item == NULL) {
+		dbg_stash("[EMPTY STASH] [CREATE] ADDING NULL ITEM TO STASH\n");
+	}
+
 	ck_stash_item_t *new_item
 		= (ck_stash_item_t *)malloc(sizeof(ck_stash_item_t));
 	if (new_item == NULL) {
@@ -681,8 +788,8 @@ int ck_add_to_stash(ck_hash_table_t *table, ck_hash_table_item_t *item)
 	new_item->next = table->stash;
 	table->stash = new_item;
 
-	dbg_ck_hash("First item in stash (now inserted): key: %.*s (size %zu)"
-	              ", value: %p\n", (int)table->stash->item->key_length,
+	dbg_ck_hash_verb("First item in stash (now inserted): key: %.*s (size"
+	              " %zu), value: %p\n", (int)table->stash->item->key_length,
 	              table->stash->item->key, table->stash->item->key_length,
 	              table->stash->item->value);
 	
@@ -739,12 +846,12 @@ ck_hash_table_t *ck_create_table(uint items)
 
 	dbg_ck("Creating hash table for %u items.\n", items);
 	dbg_ck("Exponent: %u, number of tables: %u\n ",
-		 table->table_size_exp, table->table_count);
+	       table->table_size_exp, table->table_count);
 	dbg_ck("Table size: %u items, each %zu bytes, total %zu bytes\n",
-	         hashsize(table->table_size_exp),
-	         sizeof(ck_hash_table_item_t *),
-	         hashsize(table->table_size_exp)
-	           * sizeof(ck_hash_table_item_t *));
+	       hashsize(table->table_size_exp),
+	       sizeof(ck_hash_table_item_t *),
+	       hashsize(table->table_size_exp)
+	         * sizeof(ck_hash_table_item_t *));
 
 	// create tables
 	for (uint t = 0; t < table->table_count; ++t) {
@@ -805,18 +912,6 @@ void ck_destroy_table(ck_hash_table_t **table, void (*dtor_value)(void *value),
 	}
 
 	// destroy items in stash
-//	ck_hash_table_item_t **stash =
-//	        ((ck_hash_table_item_t **)(da_get_items(&(*table)->stash)));
-//	for (uint i = 0; i < da_get_count(&(*table)->stash); ++i) {
-//		assert(stash[i] != NULL);
-//		if (dtor_value) {
-//			dtor_value(stash[i]->value);
-//		}
-//		if (delete_key != 0) {
-//			free((void *)stash[i]->key);
-//		}
-//		free((void *)stash[i]);
-//	}
 	ck_stash_item_t *item = (*table)->stash;
 	while (item != NULL) {
 		// disconnect the item
@@ -842,8 +937,6 @@ void ck_destroy_table(ck_hash_table_t **table, void (*dtor_value)(void *value),
 	for (uint t = 0; t < (*table)->table_count; ++t) {
 		free((*table)->tables[t]);
 	}
-	// destroy stash
-//	da_destroy(&(*table)->stash);
 
 	pthread_mutex_unlock(&(*table)->mtx_table);
 	// destroy mutex, assuming that here noone will lock the mutex again
@@ -884,7 +977,7 @@ void ck_table_free(ck_hash_table_t **table)
 int ck_resize_table(ck_hash_table_t *table)
 {
 	dbg_ck("Resizing hash table.\n");
-	
+
 	/*
 	 * Easiest is just to increment the exponent, resulting in doubling
 	 * the table sizes. This is not very memory-effective, but should do
@@ -900,7 +993,7 @@ int ck_resize_table(ck_hash_table_t *table)
 	ck_hash_table_item_t **tables_old[MAX_TABLES];
 	int exp_new = table->table_size_exp + 1;
 	
-	dbg_ck("New tables exponent: %d\n", exp_new);
+	dbg_ck_verb("New tables exponent: %d\n", exp_new);
 	
 	for (int t = 0; t < table->table_count; ++t) {
 		if (ck_new_table(&tables_new[t], exp_new) != 0) {
@@ -919,14 +1012,14 @@ int ck_resize_table(ck_hash_table_t *table)
 		                  * sizeof(ck_hash_table_item_t *);
 		
 		// copy the old table items
-		dbg_ck("Copying to: %p, from %p, size: %zu\n",
-		         tables_new[t], table->tables[t], old_size);
+		dbg_ck_verb("Copying to: %p, from %p, size: %zu\n",
+		            tables_new[t], table->tables[t], old_size);
 		memcpy(tables_new[t], table->tables[t], old_size);
 		// set the rest to 0
-		dbg_ck("Setting to 0 from %p, size %zu\n",
-		         tables_new[t] + hashsize(table->table_size_exp),
-		         (hashsize(exp_new) * sizeof(ck_hash_table_item_t *))
-		         - old_size);
+		dbg_ck_verb("Setting to 0 from %p, size %zu\n",
+		            tables_new[t] + hashsize(table->table_size_exp),
+		            (hashsize(exp_new) * sizeof(ck_hash_table_item_t *))
+		              - old_size);
 		memset(tables_new[t] + hashsize(table->table_size_exp), 0, 
 		       (hashsize(exp_new) * sizeof(ck_hash_table_item_t *))
 		       - old_size);
@@ -948,7 +1041,6 @@ int ck_resize_table(ck_hash_table_t *table)
 	}
 	
 	return ck_rehash(table);
-	//return 0;
 }
 
 int ck_insert_item(ck_hash_table_t *table, const char *key,
@@ -980,9 +1072,6 @@ int ck_insert_item(ck_hash_table_t *table, const char *key,
 		}
 	}
 
-	// there should be at least 2 free places
-	//assert(da_try_reserve(&table->stash, 2) == 0);
-	//da_reserve(&table->stash, 1);
 	ck_hash_table_item_t *free_place = NULL;
 	if (ck_hash_item(table, &new_item, &free_place,
 	                 table->generation) != 0) {
@@ -1104,6 +1193,8 @@ ck_hash_table_item_t *ck_remove_item(ck_hash_table_t *table, const char *key,
 
 int ck_shallow_copy(const ck_hash_table_t *from, ck_hash_table_t **to)
 {
+	dbg_ck("ck_shallow_copy()\n");
+
 	if (from == NULL || to == NULL) {
 		return -1;
 	}
@@ -1176,26 +1267,27 @@ int ck_shallow_copy(const ck_hash_table_t *from, ck_hash_table_t **to)
 			return -2;
 		}
 
-		dbg_ck("Copying stash item: %p with item %p, ", si, si->item);
-		dbg_ck("key: %.*s\n", (int)si->item->key_length, si->item->key);
+		dbg_ck_detail("Copying stash item: %p with item %p, key: %.*s"
+		              "\n", si, si->item, (int)si->item->key_length,
+		              si->item->key);
 
 		si_new->item = si->item;
 		*pos = si_new;
 		pos = &si_new->next;
 		si = si->next;
 
-
-		dbg_ck("Old stash item: %p with item %p, ", si,
-		       ((si == NULL) ? NULL : si->item));
+dbg_ck_exec_detail(
+		dbg_ck_detail("Old stash item: %p with item %p, \n", si,
+		              ((si == NULL) ? NULL : si->item));
 		if (si != NULL) {
-			dbg_ck("key: %.*s\n", (int)si->item->key_length, si->item->key);
-		} else {
-			dbg_ck("\n");
+			dbg_ck_detail("key: %.*s\n", (int)si->item->key_length,
+			              si->item->key);
 		}
-		dbg_ck("New stash item: %p with item %p, ", si_new,
-		       si_new->item);
-		dbg_ck("key: %.*s\n", (int)si_new->item->key_length, 
-		       si_new->item->key);
+		dbg_ck_detail("New stash item: %p with item %p, ", si_new,
+		              si_new->item);
+		dbg_ck_detail("key: %.*s\n", (int)si_new->item->key_length,
+		              si_new->item->key);
+);
 	}
 	
 	*pos = NULL;
@@ -1273,11 +1365,12 @@ void ck_deep_copy_cleanup(ck_hash_table_t *table, int table_count)
 
 int ck_deep_copy(ck_hash_table_t *from, ck_hash_table_t **to)
 {
+	dbg_ck("ck_deep_copy()\n");
+
 	if (from == NULL || to == NULL) {
 		return -1;
 	}
 
-	dbg_ck("Allocating new table...\n");
 	*to = (ck_hash_table_t *)malloc(sizeof(ck_hash_table_t));
 
 	if (*to == NULL) {
@@ -1341,10 +1434,12 @@ int ck_deep_copy(ck_hash_table_t *from, ck_hash_table_t **to)
 			return -2;
 		}
 
-		dbg_ck("Copying stash item: %p with item %p, ", si, si->item);
-//		dbg_ck("key: %.*s\n", (int)si->item->key_length, si->item->key);
+		dbg_ck_detail("Copying stash item: %p with item %p, ", si,
+		              si->item);
 
 		if (si->item == NULL) {
+			dbg_stash("[EMPTY STASH] [FIND] STASH ITEM IS EMPTY: "
+			        "%p (%p)\n", si, si->item);
 			si_new->item = NULL;
 			si_new->next = NULL;
 		} else {
@@ -1367,22 +1462,20 @@ int ck_deep_copy(ck_hash_table_t *from, ck_hash_table_t **to)
 		pos = &si_new->next;
 		si = si->next;
 
-dbg_ck_exec(
-		dbg_ck("Old stash item: %p with item %p, ", si,
-		       ((si == NULL) ? NULL : si->item));
+dbg_ck_exec_detail(
+		dbg_ck_detail("Old stash item: %p with item %p, \n", si,
+		              ((si == NULL) ? NULL : si->item));
 		if (si != NULL && si->item != NULL) {
-			dbg_ck("key: %.*s\n", (int)si->item->key_length,
-			       si->item->key);
-		} else {
-			dbg_ck("\n");
+			dbg_ck_detail("key: %.*s\n", (int)si->item->key_length,
+			              si->item->key);
 		}
-		dbg_ck("New stash item: %p with item %p, ", si_new,
-		       (si_new) ? si_new->item : NULL);
+		dbg_ck_detail("New stash item: %p with item %p, ", si_new,
+		              (si_new) ? si_new->item : NULL);
 
 		assert(si_new != NULL);
 		assert(si_new->item != NULL);
-		dbg_ck("key: %.*s\n", (int)si_new->item->key_length,
-		       si_new->item->key);
+		dbg_ck_detail("key: %.*s\n", (int)si_new->item->key_length,
+		              si_new->item->key);
 );
 	}
 
@@ -1446,13 +1539,13 @@ int ck_rehash(ck_hash_table_t *table)
 
 	do {
 		// 1) Rehash items from stash
-		dbg_ck_rehash("Rehashing items from stash.\n");
+		dbg_ck_hash_verb("Rehashing items from stash.\n");
 		ck_stash_item_t *item = table->stash;
 		ck_stash_item_t **item_place = &table->stash;
 		// terminate when at the end; this way the newly added items
 		// (added to the beginning) will be properly ignored
 		while (item != NULL) {
-			dbg_ck_rehash("Rehashing item with "
+			dbg_ck_hash_detail("Rehashing item with "
 			  "key (length %zu): %.*s, generation: %hu, "
 			  "table generation: %hu.\n", item->item->key_length,
 			  (int)item->item->key_length, item->item->key,
@@ -1481,6 +1574,8 @@ int ck_rehash(ck_hash_table_t *table)
 				assert(item->item == NULL);
 				// and the item should be hashed too
 //				assert(table->hashed == NULL);
+				dbg_stash("[EMPTY STASH] [CREATE] Created empty"
+				          " item: %p (%p)\n", item, item->item);
 
 				// fix the pointer from the previous hash item
 				*item_place = item->next;
@@ -1501,12 +1596,11 @@ int ck_rehash(ck_hash_table_t *table)
 		// which will be put to the stash
 		ck_hash_table_item_t *free = NULL;
 		assert(table->hashed == NULL);
-//		ck_hash_table_item_t *old = table->hashed;
 
 		for (uint t = 0; t < table->table_count; ++t) {
 			uint rehashed = 0;
 
-			dbg_ck_rehash("Rehashing table %d.\n", t);
+			dbg_ck_hash_verb("Rehashing table %d.\n", t);
 
 			while (rehashed < hashsize(table->table_size_exp)) {
 
@@ -1516,13 +1610,13 @@ int ck_rehash(ck_hash_table_t *table)
 				    || !(EQUAL_GENERATIONS(
 				          table->tables[t][rehashed]->timestamp,
 				          table->generation))) {
-					dbg_ck_rehash("Skipping item.\n");
+					dbg_ck_hash_detail("Skipping item.\n");
 					++rehashed;
 					continue;
 				}
 
-				dbg_ck_rehash("Rehashing item with hash %u, "
-				  "key (length %zu): %.*s, generation: %hu, "
+				dbg_ck_hash_detail("Rehashing item with hash %u"
+				  ", key (length %zu): %.*s, generation: %hu, "
 				  "table generation: %hu.\n", rehashed,
 				  table->tables[t][rehashed]->key_length,
 				  (int)(table->tables[t][rehashed]->key_length),
@@ -1537,8 +1631,8 @@ int ck_rehash(ck_hash_table_t *table)
 				// get rehashed again
 				ck_clear_item(&table->tables[t][rehashed]);
 
-				dbg_ck_rehash("Table generation: %hu, next "
-				            "generation: %hu.\n",
+				dbg_ck_hash_detail("Table generation: %hu, next"
+				            " generation: %hu.\n",
 				            GET_GENERATION(table->generation),
 				            NEXT_GENERATION(table->generation));
 
@@ -1546,8 +1640,8 @@ int ck_rehash(ck_hash_table_t *table)
 				     NEXT_GENERATION(table->generation)) != 0) {
 					// loop occured
 					dbg_ck_hash("Hashing entered a loop."
-						      "\n");
-					dbg_ck_rehash("Item with key %.*s "
+					            "\n");
+					dbg_ck_hash_verb("Item with key %.*s "
 					  "inserted into the free slot.\n",
 					  free->key_length, free->key);
 
@@ -1561,6 +1655,12 @@ int ck_rehash(ck_hash_table_t *table)
 							free_stash_items;
 						free_stash_items = item->next;
 
+						if (free == NULL) {
+							dbg_stash("[EMPTY STASH] "
+							 "[CREATE] STORING NULL"
+							 " in the stash\n");
+						}
+
 						item->item = free;
 						item->next = table->stash;
 						table->stash = item;
@@ -1579,15 +1679,15 @@ int ck_rehash(ck_hash_table_t *table)
 			}
 		}
 
-		dbg_ck_rehash("Old table generation: %u\n",
-		                GET_GENERATION(table->generation));
+		dbg_ck_hash("Old table generation: %u\n",
+		            GET_GENERATION(table->generation));
 		// rehashing completed, switch generation of the table
 		SET_NEXT_GENERATION(&table->generation);
-		dbg_ck_rehash("New table generation: %u\n",
-		                GET_GENERATION(table->generation));
+		dbg_ck_hash("New table generation: %u\n",
+		            GET_GENERATION(table->generation));
 		// generate new hash functions for the old generation
-		dbg_ck_rehash("Generating coeficients for generation: %u\n",
-		                NEXT_GENERATION(table->generation));
+		dbg_ck_hash("Generating coeficients for generation: %u\n",
+		            NEXT_GENERATION(table->generation));
 		us_next(&table->hash_system,
 		        NEXT_GENERATION(table->generation));
 
@@ -1608,248 +1708,6 @@ int ck_rehash(ck_hash_table_t *table)
 	return 0;
 }
 
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief Rehashes the whole table.
- *
- * \param table Hash table to be rehashed.
- *
- * \note While rehashing no item should be inserted as it will result in a
- *       deadlock.
- *
- * \retval 0 No error.
- * \retval -1 Rehashing failed. Some items may have been already moved and the
- *            rehashing flag remains set.
- *
- * \todo What if the stash is reallocated during ck_hash_item()? We'd be using
- *       the old stash for saving items! The old stash would not get deallocated
- *       (due to RCU - maybe put some rcu_read_lock() here), but the item
- *       would not be saved into the new stash!
- *       Maybe add a function for getting a pointer to particular item from
- *       the dynamic array and protect it using rcu_read_lock().
- *       Other option: Do not use pointer to an item in stash in the call to
- *       ck_hash_item(). Use some new place & put the item to the stash
- *       afterwards, protecting it using rcu_read_lock() and rcu_assign_pointer.
- */
-//int ck_rehash(ck_hash_table_t *table)
-//{
-//	dbg_ck_rehash("Rehashing items in table.\n");
-//	SET_REHASHING_ON(&table->generation);
-
-//	// we already have functions for the next generation, begin rehashing
-//	// we wil use the last item in the buffer as free cell for hashing
-//	assert(da_try_reserve(&table->stash, 1) == 0);
-//	ck_hash_table_item_t *old = (ck_hash_table_item_t *)
-//	                          (malloc(sizeof(ck_hash_table_item_t)));
-
-//	do {
-//		dbg_ck_hash("Rehash!\n");
-
-//		if (da_get_count(&table->stash) > STASH_SIZE) {
-//			dbg_ck_hash("STASH RESIZED!!! (new stash size: %d)\n",
-//			              da_get_count(&table->stash));
-//		}
-
-//		// rehash items from stash, starting from the last old item
-//		int stash_i = da_get_count(&table->stash) - 1;
-//		while (stash_i >= 0) {
-//			// if item's generation is the new generation, skip
-//			if (STASH_ITEMS(&table->stash)[stash_i] == NULL
-//			    || !(EQUAL_GENERATIONS(STASH_ITEMS(&table->stash)
-//			                            [stash_i]->timestamp,
-//			                            table->generation))) {
-//				dbg_ck_rehash("Skipping item.\n");
-//				--stash_i;
-//				continue;
-//			}
-
-//			dbg_ck_rehash("Rehashing item from buffer position %u"
-//			                ", key (length %u): %.*s, generation: "
-//			                "%hu, table generation: %hu.\n",
-//			   stash_i,
-//			   STASH_ITEMS(&table->stash)[stash_i]->key_length,
-//			   (int)STASH_ITEMS(&table->stash)[stash_i]->key_length,
-//			   STASH_ITEMS(&table->stash)[stash_i]->key,
-//			   GET_GENERATION(
-//				STASH_ITEMS(&table->stash)[stash_i]->timestamp),
-//			   GET_GENERATION(table->generation));
-
-//			// otherwise copy the item for rehashing
-//			ck_put_item(&old, STASH_ITEMS(&table->stash)[stash_i]);
-//			// clear the place so that this item will not get
-//			// rehashed again
-//			ck_clear_item(&STASH_ITEMS(&table->stash)[stash_i]);
-//			da_release(&table->stash, 1);
-
-//			// there should be at least one place in the stash
-//			assert(da_try_reserve(&table->stash, 1) == 0);
-//			da_reserve(&table->stash, 1);
-
-//			assert(STASH_ITEMS(&table->stash)[stash_i] == NULL);
-
-//			// and start rehashing
-//			if (ck_hash_item(table, &old,
-//			             &STASH_ITEMS(&table->stash)[stash_i],
-//			             NEXT_GENERATION(table->generation)) != 0) {
-//				// loop occured
-//				dbg_ck_hash("Hashing entered a loop.\n");
-
-//				dbg_ck_rehash("Item with key %.*s inserted "
-//					"into the stash on position %d.\n",
-//					STASH_ITEMS(&table->stash)
-//						[stash_i]->key_length,
-//					STASH_ITEMS(&table->stash)
-//						[stash_i]->key,
-//					da_get_count(&table->stash));
-
-//				// hashing unsuccessful, the item was inserted
-//				// into the stash
-//				da_occupy(&table->stash, 1);
-//				assert(STASH_ITEMS(&table->stash)[stash_i]
-//				       != NULL);
-
-//				// if only one place left, resize the stash
-//				// TODO: Why???
-//				if (da_reserve(&table->stash, 2) < 0) {
-//					// stash could not be resized => !!!
-//					dbg_ck_hash("Failed to rehash items "
-//					              "from "
-//					  "table, no other rehash possible!\n");
-//					// so rollback
-//					ck_rollback_rehash(table);
-//					// clear the 'old' item
-//					ck_clear_item(&old);
-//					return -1;
-//				}
-//			}
-
-//			// clear the 'old' item
-//			ck_clear_item(&old);
-//			// decrement the index
-//			--stash_i;
-//		}
-
-//		uint i = 0;
-//		while (i < da_get_count(&table->stash)) {
-//			assert(STASH_ITEMS(&table->stash)[i] != NULL);
-//			++i;
-//		}
-//		dbg_ck_hash("OK\n");
-//		assert(da_try_reserve(&table->stash, 1) == 0);
-//		assert(STASH_ITEMS(&table->stash)[da_get_count(&table->stash)]
-//		       == NULL);
-
-//		// rehash items from hash tables
-//		for (uint t = TABLE_FIRST;
-//		     t <= TABLE_LAST(table->table_count); ++t) {
-//			dbg_ck_rehash("Rehashing items from table %d.\n",
-//			                t + 1);
-//			uint rehashed = 0;
-
-//			while (rehashed < hashsize(table->table_size_exp)) {
-
-//				// if item's generation is the new generation,
-//				// skip
-//				if (table->tables[t][rehashed] == NULL
-//				    || !(EQUAL_GENERATIONS(
-//				          table->tables[t][rehashed]->timestamp,
-//				          table->generation))) {
-//					dbg_ck_rehash("Skipping item.\n");
-//					++rehashed;
-//					continue;
-//				}
-
-//				dbg_ck_rehash("Rehashing item with hash %u, "
-//				  "key (length %u): %.*s, generation: %hu, "
-//				  "table generation: %hu.\n", rehashed,
-//				  table->tables[t][rehashed]->key_length,
-//				  (int)(table->tables[t][rehashed]->key_length),
-//				  table->tables[t][rehashed]->key,
-//				  GET_GENERATION(
-//					table->tables[t][rehashed]->timestamp),
-//				  GET_GENERATION(table->generation));
-
-//				// otherwise copy the item for rehashing
-//				ck_put_item(&old, table->tables[t][rehashed]);
-//				// clear the place so that this item will not
-//				// get rehashed again
-//				ck_clear_item(&table->tables[t][rehashed]);
-
-//				dbg_ck_rehash("Table generation: %hu, next "
-//				            "generation: %hu.\n",
-//				            GET_GENERATION(table->generation),
-//				            NEXT_GENERATION(table->generation));
-
-//				// and start rehashing
-//				assert(&old != &STASH_ITEMS(&table->stash)[
-//				               da_get_count(&table->stash)]);
-//				assert(da_try_reserve(&table->stash, 1) == 0);
-//				da_reserve(&table->stash, 1);
-
-//				if (ck_hash_item(table, &old,
-//				     &STASH_ITEMS(&table->stash)[
-//				       da_get_count(&table->stash)],
-//				     NEXT_GENERATION(table->generation)) != 0) {
-//					// loop occured
-//					dbg_ck_hash("Hashing entered a loop."
-//						      "\n");
-//					dbg_ck_rehash("Item with key %.*s "
-//					  "inserted into the stash on position "
-//					  "%d.\n", STASH_ITEMS(&table->stash)[
-//					      da_get_count(&table->stash)]
-//					         ->key_length,
-//					  STASH_ITEMS(&table->stash)[
-//					      da_get_count(&table->stash)]->key,
-//					  da_get_count(&table->stash));
-
-//					assert(STASH_ITEMS(&table->stash)[
-//					  da_get_count(&table->stash)] != NULL);
-//					// loop occured, the item is already at
-//					// its new place in the buffer, so just
-//					// increment the index
-//					da_occupy(&table->stash, 1);
-
-//					// if only one place left, resize the
-//					// stash TODO: Why?
-//					if (da_reserve(&table->stash, 2) < 0) {
-//						// stash could not be resized
-//						dbg_ck_hash("Failed to rehash"
-//						  " items from table, no other "
-//						  "rehash possible!\n");
-//						// so rollback
-//						ck_rollback_rehash(table);
-//						// clear the 'old' item
-//						ck_clear_item(&old);
-//						return -1;
-//					}
-//				}
-//				++rehashed;
-//			}
-//		}
-
-//		dbg_ck_rehash("Old table generation: %u\n",
-//		                GET_GENERATION(table->generation));
-//		// rehashing completed, switch generation of the table
-//		SET_NEXT_GENERATION(&table->generation);
-//		dbg_ck_rehash("New table generation: %u\n",
-//		                GET_GENERATION(table->generation));
-//		// generate new hash functions for the old generation
-//		dbg_ck_rehash("Generating coeficients for generation: %u\n",
-//		                NEXT_GENERATION(table->generation));
-//		us_next(NEXT_GENERATION(table->generation));
-
-//		// repeat rehashing while there are more items in the stash than
-//		// its initial size
-//		if (da_get_count(&table->stash) > STASH_SIZE) {
-//			dbg_ck_rehash("Rehashing again!\n");
-//		}
-//	} while (da_get_count(&table->stash) > STASH_SIZE);
-
-//	SET_REHASHING_OFF(&table->generation);
-
-//	return 0;
-//}
-
 /*----------------------------------------------------------------------------*/
 
 void ck_dump_table(const ck_hash_table_t *table)
@@ -1872,15 +1730,6 @@ void ck_dump_table(const ck_hash_table_t *table)
 	}
 
 	dbg_ck("Stash:\n");
-//	for (i = 0; i < da_get_count(&table->stash); ++i) {
-//		dbg_ck("Index: %u, Key: %.*s Value: %p.\n", i,
-//		         ((ck_hash_table_item_t **)
-//		             da_get_items(&table->stash))[i]->key_length,
-//		         ((ck_hash_table_item_t **)
-//		             da_get_items(&table->stash))[i]->key,
-//		         ((ck_hash_table_item_t **)
-//		             da_get_items(&table->stash))[i]->value);
-//	}
 	ck_stash_item_t *item = table->stash;
 	while (item != NULL) {
 		dbg_ck("Hash: %u, Key: %.*s, Value: %p.\n", i,
