@@ -21,20 +21,26 @@
 #include <time.h>
 
 #include "common.h"
+#include "common/base64.h"
 #include "tsig.h"
 #include "tsig-op.h"
 #include "util/wire.h"
-#include "libknot/util/conv.h"
-#include "util/error.h"
 #include "util/debug.h"
 #include "consts.h"
 
 
 const int KNOT_TSIG_MAX_DIGEST_SIZE = 64;    // size of HMAC-SHA512 digest
-
+const uint16_t KNOT_TSIG_FUDGE_DEFAULT = 300;  // default Fudge value
+enum b64_const {
+	B64BUFSIZE = 65535
+};
 
 static int knot_tsig_check_algorithm(const knot_rrset_t *tsig_rr)
 {
+	if (tsig_rr == NULL) {
+		return KNOT_EINVAL;
+	}
+	
 	const knot_dname_t *alg_name = tsig_rdata_alg_name(tsig_rr);
 	if (!alg_name) {
 		return KNOT_EMALF;
@@ -53,6 +59,10 @@ static int knot_tsig_check_algorithm(const knot_rrset_t *tsig_rr)
 static int knot_tsig_check_key(const knot_rrset_t *tsig_rr,
                                const knot_key_t *tsig_key)
 {
+	if (tsig_rr == NULL || tsig_key == NULL) {
+		return KNOT_EINVAL;
+	}
+	
 	const knot_dname_t *tsig_name = knot_rrset_owner(tsig_rr);
 	if (!tsig_name) {
 		return KNOT_EMALF;
@@ -80,7 +90,7 @@ static int knot_tsig_compute_digest(const uint8_t *wire, size_t wire_len,
 {
 	if (!wire || !digest || !digest_len || !key) {
 		dbg_tsig("TSIG: digest: bad args.\n");
-		return KNOT_EBADARG;
+		return KNOT_EINVAL;
 	}
 
 	if (!key->name) {
@@ -94,28 +104,27 @@ static int knot_tsig_compute_digest(const uint8_t *wire, size_t wire_len,
 		return KNOT_TSIG_EBADSIG;
 	}
 
-	/* Create digest, using length of the algorithm. */
-//	*digest = malloc(sizeof(uint8_t) * tsig_alg_digest_length(tsig_alg));
-//	if (!digest) {
-//		ERR_ALLOC_FAILED;
-//		return KNOT_ENOMEM;
-//	}
-
 	/* Decode key from Base64. */
 	char decoded_key[B64BUFSIZE];
-
-	int decoded_key_size = b64_pton(key->secret, (uint8_t *)decoded_key,
-					B64BUFSIZE);
+	memset(decoded_key, 0, sizeof(decoded_key));
+	
+	size_t decoded_key_size = B64BUFSIZE;
+	int ret = base64_decode(key->secret, strlen(key->secret),
+	                        decoded_key,
+	                        &decoded_key_size);
+	if (ret != 1) {
+		dbg_tsig("TSIG: New decode function failed! (%d)\n", ret);
+		return KNOT_ERROR;
+	}
+	
 	if (decoded_key_size < 0) {
 		dbg_tsig("TSIG: Could not decode Base64\n");
 		return KNOT_ERROR;
 	}
-
+	
 	dbg_tsig_detail("TSIG: decoded key size: %d\n", decoded_key_size);
-	dbg_tsig_detail("TSIG: decoded key: '%*s'\n", decoded_key_size, decoded_key);
-
-//	dbg_tsig_detail("TSIG: using this wire for digest calculation\n");
-//	dbg_tsig_hex_detail(wire, wire_len);
+	dbg_tsig_detail("TSIG: decoded key:\n");
+	dbg_tsig_hex_detail(decoded_key, decoded_key_size);
 	dbg_tsig_detail("Wire for signing is %zu bytes long.\n", wire_len);
 
 	/* Compute digest. */
@@ -152,7 +161,8 @@ static int knot_tsig_check_time_signed(const knot_rrset_t *tsig_rr,
                                        uint64_t prev_time_signed)
 {
 	if (!tsig_rr) {
-		return KNOT_EBADARG;
+		dbg_tsig("TSIG: check_time_signed: NULL argument.\n");
+		return KNOT_EINVAL;
 	}
 
 	/* Get the time signed and fudge values. */
@@ -184,50 +194,42 @@ static int knot_tsig_check_time_signed(const knot_rrset_t *tsig_rr,
 	return KNOT_EOK;
 }
 
-static int knot_tsig_write_tsig_timers(uint8_t *wire, 
-                                       const knot_rrset_t *tsig_rr)
-{
-	// put time signed
-	knot_wire_write_u48(wire, tsig_rdata_time_signed(tsig_rr));
-	
-	// put fudge
-	knot_wire_write_u16(wire + 6, tsig_rdata_fudge(tsig_rr));
-	
-	return KNOT_EOK;
-}
-
 static int knot_tsig_write_tsig_variables(uint8_t *wire,
-                                         const knot_rrset_t *tsig_rr)
+                                          const knot_rrset_t *tsig_rr)
 {
+	if (wire == NULL || tsig_rr == NULL) {
+		dbg_tsig("TSIG: write tsig variables: NULL arguments.\n");
+		return KNOT_EINVAL;
+	}
+	
 	/* Copy TSIG variables - starting with key name. */
 	const knot_dname_t *tsig_owner = knot_rrset_owner(tsig_rr);
 	if (!tsig_owner) {
 		dbg_tsig("TSIG: write variables: no owner.\n");
-		return KNOT_EBADARG;
+		return KNOT_EINVAL;
 	}
 
 	int offset = 0;
 
 	memcpy(wire + offset, knot_dname_name(tsig_owner),
 	       sizeof(uint8_t) * knot_dname_size(tsig_owner));
-	dbg_tsig("TSIG: write variables: written owner (tsig alg): \n");
-	         /*knot_rrset_class(tsig_rr));*/
-	dbg_tsig_hex_detail(wire + offset, knot_dname_size(tsig_owner));
+	dbg_tsig_verb("TSIG: write variables: written owner (tsig alg): \n");
+	dbg_tsig_hex_verb(wire + offset, knot_dname_size(tsig_owner));
 	offset += knot_dname_size(tsig_owner);
 
 	/*!< \todo which order? */
 
 	/* Copy class. */
 	knot_wire_write_u16(wire + offset, knot_rrset_class(tsig_rr));
-	dbg_tsig("TSIG: write variables: written CLASS: %u - ",
-	         knot_rrset_class(tsig_rr));
+	dbg_tsig_verb("TSIG: write variables: written CLASS: %u - ",
+	               knot_rrset_class(tsig_rr));
 	dbg_tsig_hex_detail(wire + offset, sizeof(uint16_t));
 	offset += sizeof(uint16_t);
 
 	/* Copy TTL - always 0. */
 	knot_wire_write_u32(wire + offset, knot_rrset_ttl(tsig_rr));
-	dbg_tsig("TSIG: write variables: written TTL: %u - ",
-	         knot_rrset_ttl(tsig_rr));
+	dbg_tsig_verb("TSIG: write variables: written TTL: %u - ",
+	              knot_rrset_ttl(tsig_rr));
 	dbg_tsig_hex_detail(wire + offset, sizeof(uint32_t));
 	offset += sizeof(uint32_t);
 
@@ -235,30 +237,41 @@ static int knot_tsig_write_tsig_variables(uint8_t *wire,
 	const knot_dname_t *alg_name = tsig_rdata_alg_name(tsig_rr);
 	if (!alg_name) {
 		dbg_tsig("TSIG: write variables: no algorithm name.\n");
-		return KNOT_EBADARG;
+		return KNOT_EINVAL;
 	}
-//	alg_name = knot_dname_new_from_str("HMAC-MD5.SIG-ALG.REG.INT.",
-					   //strlen("HMAC-MD5.SIG-ALG.REG.INT."),
-					   //NULL);
 
-	memcpy(wire + offset, knot_dname_name(alg_name),
-	       sizeof(uint8_t) * knot_dname_size(alg_name));
+	/* The algorithm name must be in canonical form, i.e. in lowercase. */
+	if (knot_dname_to_lower_copy(alg_name, (char *)(wire + offset),
+	        knot_dname_size(alg_name)) != KNOT_EOK)
+	{
+		dbg_tsig("TSIG: write variables: cannot convert algorithm "
+		         "to lowercase.\n");
+		return KNOT_EINVAL;
+	}
+
+#if defined(KNOT_TSIG_DEBUG) && defined(DEBUG_ENABLE_VERBOSE)
+//	char *_algstr = knot_dname_to_str(alg_name);
+	dbg_tsig_verb("TSIG: write variables: written alg name: %s\n",
+	              wire + offset);
+//	free(_algstr);
+#endif
+
+//	memcpy(wire + offset, knot_dname_name(alg_name),
+//	       sizeof(uint8_t) * knot_dname_size(alg_name));
 	offset += knot_dname_size(alg_name);
-	dbg_tsig_detail("TSIG: write variables: written alg name: %s\n",
-		 knot_dname_to_str(alg_name));
 
 	/* Following data are written in network order. */
 	/* Time signed. */
 	knot_wire_write_u48(wire + offset, tsig_rdata_time_signed(tsig_rr));
 	offset += 6;
-	dbg_tsig_detail("TSIG: write variables: time signed: %llu - ",
-		        tsig_rdata_time_signed(tsig_rr));
+	dbg_tsig_verb("TSIG: write variables: time signed: %llu \n",
+	              tsig_rdata_time_signed(tsig_rr));
 	dbg_tsig_hex_detail(wire + offset - 6, 6);
 	/* Fudge. */
 	knot_wire_write_u16(wire + offset, tsig_rdata_fudge(tsig_rr));
 	offset += sizeof(uint16_t);
-	dbg_tsig_detail("TSIG: write variables: fudge: %hu\n",
-		 tsig_rdata_fudge(tsig_rr));
+	dbg_tsig_verb("TSIG: write variables: fudge: %hu\n",
+	              tsig_rdata_fudge(tsig_rr));
 	/* TSIG error. */
 	knot_wire_write_u16(wire + offset, tsig_rdata_error(tsig_rr));
 	offset += sizeof(uint16_t);
@@ -268,7 +281,7 @@ static int knot_tsig_write_tsig_variables(uint8_t *wire,
 	const uint8_t *other_data = tsig_rdata_other_data(tsig_rr);
 	if (!other_data) {
 		dbg_tsig("TSIG: write variables: no other data.\n");
-		return KNOT_EBADARG;
+		return KNOT_EINVAL;
 	}
 
 	/*
@@ -279,7 +292,7 @@ static int knot_tsig_write_tsig_variables(uint8_t *wire,
 	offset += sizeof(uint16_t);
 
 	/* Skip the length. */
-	dbg_tsig_detail("Copying other data.\n");
+	dbg_tsig_verb("Copying other data.\n");
 	memcpy(wire + offset, other_data, other_data_length);
 
 	return KNOT_EOK;
@@ -288,7 +301,14 @@ static int knot_tsig_write_tsig_variables(uint8_t *wire,
 static int knot_tsig_wire_write_timers(uint8_t *wire,
                                        const knot_rrset_t *tsig_rr)
 {
+	if (wire == NULL || tsig_rr == NULL) {
+		dbg_tsig("TSIG: write timers: NULL arguments.\n");
+		return KNOT_EINVAL;
+	}
+	
+	//write time signed
 	knot_wire_write_u48(wire, tsig_rdata_time_signed(tsig_rr));
+	//write fudge
 	knot_wire_write_u16(wire + 6, tsig_rdata_fudge(tsig_rr));
 
 	return KNOT_EOK;
@@ -303,26 +323,19 @@ static int knot_tsig_create_sign_wire(const uint8_t *msg, size_t msg_len,
 {
 	if (!msg || !key || digest_len == NULL) {
 		dbg_tsig("TSIG: create wire: bad args.\n");
-		return KNOT_EBADARG;
+		return KNOT_EINVAL;
 	}
 
 	/* Create tmp TSIG. */
 	int ret = KNOT_EOK;
-//	knot_rrset_t *tmp_tsig =
-//		knot_rrset_new(key->name, KNOT_RRTYPE_TSIG, KNOT_CLASS_ANY, 0);
-//	if (!tmp_tsig) {
-//		return KNOT_ENOMEM;
-//	}
-
-//	tsig_rdata_store_current_time(tmp_tsig);
 
 	/*
 	 * Create tmp wire, it should contain message
 	 * plus request mac plus tsig varibles.
 	 */
-	dbg_tsig("Counting wire size: %zu, %zu, %zu.\n",
-	         msg_len, request_mac_len,
-	         tsig_rdata_tsig_variables_length(tmp_tsig));
+	dbg_tsig_verb("Counting wire size: %zu, %zu, %zu.\n",
+	              msg_len, request_mac_len,
+	              tsig_rdata_tsig_variables_length(tmp_tsig));
 	size_t wire_len = sizeof(uint8_t) *
 			(msg_len + request_mac_len + ((request_mac_len > 0)
 			 ? 2 : 0) +
@@ -339,23 +352,21 @@ static int knot_tsig_create_sign_wire(const uint8_t *msg, size_t msg_len,
 
 	/* Copy the request MAC - should work even if NULL. */
 	if (request_mac_len > 0) {
-		dbg_tsig_detail("Copying request MAC size\n");
+		dbg_tsig_verb("Copying request MAC size\n");
 		knot_wire_write_u16(pos, request_mac_len);
 		pos += 2;
 	}
-	dbg_tsig("Copying request mac.\n");
+	dbg_tsig_verb("Copying request mac.\n");
 	memcpy(pos, request_mac, sizeof(uint8_t) * request_mac_len);
-	dbg_tsig_detail("TSIG: create wire: request mac: ");
+	dbg_tsig_detail("TSIG: create wire: request mac:\n");
 	dbg_tsig_hex_detail(pos, request_mac_len);
 	pos += request_mac_len;
 	/* Copy the original message. */
-	dbg_tsig("Copying original message.\n");
+	dbg_tsig_verb("Copying original message.\n");
 	memcpy(pos, msg, msg_len);
-	dbg_tsig_detail("TSIG: create wire: original message: \n");
-	//dbg_tsig_hex_detail(pos, msg_len);
 	pos += msg_len;
 	/* Copy TSIG variables. */
-	dbg_tsig("Writing TSIG variables.\n");
+	dbg_tsig_verb("Writing TSIG variables.\n");
 	ret = knot_tsig_write_tsig_variables(pos, tmp_tsig);
 	if (ret != KNOT_EOK) {
 		dbg_tsig("TSIG: create wire: failed to write TSIG "
@@ -365,8 +376,7 @@ static int knot_tsig_create_sign_wire(const uint8_t *msg, size_t msg_len,
 	}
 
 	/* Compute digest. */
-	ret = knot_tsig_compute_digest(wire, wire_len,
-	                               digest, digest_len, key);
+	ret = knot_tsig_compute_digest(wire, wire_len, digest, digest_len, key);
 	if (ret != KNOT_EOK) {
 		dbg_tsig("TSIG: create wire: failed to compute digest: %s\n",
 		         knot_strerror(ret));
@@ -375,20 +385,8 @@ static int knot_tsig_create_sign_wire(const uint8_t *msg, size_t msg_len,
 		return ret;
 	}
 
-//	assert(digest_tmp_len > 0);
 	free(wire);
 
-//	if (digest_tmp_len > *digest_len) {
-//		*digest_len = 0;
-//		return KNOT_ESPACE;
-//	}
-
-//	knot_rrset_deep_free(&tmp_tsig, 1, 1, 1);
-
-	// everything went ok, save the digest to the output parameter
-//	memcpy(digest, digest_tmp, digest_tmp_len);
-//	*digest_len = digest_tmp_len;
-
 	return KNOT_EOK;
 }
 
@@ -401,7 +399,7 @@ static int knot_tsig_create_sign_wire_next(const uint8_t *msg, size_t msg_len,
 {
 	if (!msg || !key || digest_len == NULL) {
 		dbg_tsig("TSIG: create wire: bad args.\n");
-		return KNOT_EBADARG;
+		return KNOT_EINVAL;
 	}
 
 	/* Create tmp TSIG. */
@@ -411,9 +409,9 @@ static int knot_tsig_create_sign_wire_next(const uint8_t *msg, size_t msg_len,
 	 * Create tmp wire, it should contain message
 	 * plus request mac plus tsig varibles.
 	 */
-	dbg_tsig("Counting wire size: %zu, %zu, %zu.\n",
-	         msg_len, prev_mac_len,
-	         tsig_rdata_tsig_timers_length());
+	dbg_tsig_verb("Counting wire size: %zu, %zu, %zu.\n",
+	              msg_len, prev_mac_len,
+	              tsig_rdata_tsig_timers_length());
 	size_t wire_len = sizeof(uint8_t) *
 	                (msg_len + prev_mac_len +
 			tsig_rdata_tsig_timers_length() + 2);
@@ -426,27 +424,24 @@ static int knot_tsig_create_sign_wire_next(const uint8_t *msg, size_t msg_len,
 	memset(wire, 0, wire_len);
 
 	/* Copy the request MAC - should work even if NULL. */
-	dbg_tsig("Copying request mac size.\n");
+	dbg_tsig_verb("Copying request mac size.\n");
 	knot_wire_write_u16(wire, prev_mac_len);
-	dbg_tsig("Copying request mac.\n");
+	dbg_tsig_verb("Copying request mac.\n");
 	memcpy(wire + 2, prev_mac, sizeof(uint8_t) * prev_mac_len);
-	dbg_tsig_detail("TSIG: create wire: request mac: ");
+	dbg_tsig_detail("TSIG: create wire: request mac:\n");
 	dbg_tsig_hex_detail(wire + 2, prev_mac_len);
 	/* Copy the original message. */
-	dbg_tsig("Copying original message.\n");
+	dbg_tsig_verb("Copying original message.\n");
 	memcpy(wire + prev_mac_len + 2, msg, msg_len);
-	dbg_tsig_detail("TSIG: create wire: original message: \n");
-	//dbg_tsig_hex_detail(wire + prev_mac_len, msg_len);
 	/* Copy TSIG variables. */
 	
-	dbg_tsig("Writing TSIG timers.\n");
-	ret = knot_tsig_write_tsig_timers(wire + prev_mac_len + msg_len + 2, 
+	dbg_tsig_verb("Writing TSIG timers.\n");
+	ret = knot_tsig_wire_write_timers(wire + prev_mac_len + msg_len + 2, 
 	                                  tmp_tsig);
-//	ret = knot_tsig_write_tsig_variables(wire + prev_mac_len + msg_len,
-//	                                     tmp_tsig);
 	if (ret != KNOT_EOK) {
 		dbg_tsig("TSIG: create wire: failed to write TSIG "
 		         "timers: %s\n", knot_strerror(ret));
+		free(wire);
 		return ret;
 	}
 
@@ -457,6 +452,7 @@ static int knot_tsig_create_sign_wire_next(const uint8_t *msg, size_t msg_len,
 		dbg_tsig("TSIG: create wire: failed to compute digest: %s\n",
 		         knot_strerror(ret));
 		*digest_len = 0;
+		free(wire);
 		return ret;
 	}
 
@@ -473,12 +469,12 @@ int knot_tsig_sign(uint8_t *msg, size_t *msg_len,
                    uint64_t request_time_signed)
 {
 	if (!msg || !msg_len || !key || digest == NULL || digest_len == NULL) {
-		return KNOT_EBADARG;
+		return KNOT_EINVAL;
 	}
 
 	knot_dname_t *key_name_copy = knot_dname_deep_copy(key->name);
 	if (!key_name_copy) {
-		dbg_tsig_detail("TSIG: key_name_copy = NULL\n");
+		dbg_tsig("TSIG: key_name_copy = NULL\n");
 		return KNOT_ENOMEM;
 	}
 
@@ -488,14 +484,14 @@ int knot_tsig_sign(uint8_t *msg, size_t *msg_len,
 	/* Should be retained by rrsig or freed, release. */
 	knot_dname_release(key_name_copy);
 	if (!tmp_tsig) {
-		dbg_tsig_detail("TSIG: tmp_tsig = NULL\n");
+		dbg_tsig("TSIG: tmp_tsig = NULL\n");
 		return KNOT_ENOMEM;
 	}
 
 	/* Create rdata for TSIG RR. */
 	knot_rdata_t *rdata = knot_rdata_new();
 	if (!rdata) {
-		dbg_tsig_detail("TSIG: rdata = NULL\n");
+		dbg_tsig("TSIG: rdata = NULL\n");
 		knot_rrset_free(&tmp_tsig);
 		return KNOT_ENOMEM;
 	}
@@ -510,7 +506,7 @@ int knot_tsig_sign(uint8_t *msg, size_t *msg_len,
 	knot_rdata_item_t *items =
 		malloc(sizeof(knot_rdata_item_t) * desc->length);
 	if (!items) {
-		dbg_tsig_detail("TSIG: items = NULL\n");
+		dbg_tsig("TSIG: items = NULL\n");
 		ERR_ALLOC_FAILED;
 		knot_rrset_free(&tmp_tsig);
 		knot_rdata_free(&rdata);
@@ -521,7 +517,8 @@ int knot_tsig_sign(uint8_t *msg, size_t *msg_len,
 
 	int ret = knot_rdata_set_items(rdata, items, desc->length);
 	if (ret != KNOT_EOK) {
-		dbg_tsig_detail("TSIG: rdata_set_items returned %s\n", knot_strerror(ret));
+		dbg_tsig("TSIG: rdata_set_items returned %s\n",
+		         knot_strerror(ret));
 		free(items);
 		knot_rrset_free(&tmp_tsig);
 		knot_rdata_free(&rdata);
@@ -554,7 +551,7 @@ int knot_tsig_sign(uint8_t *msg, size_t *msg_len,
 		tsig_rdata_set_other_data(tmp_tsig, 0, 0);
 	}
 
-	tsig_rdata_set_fudge(tmp_tsig, 300);   /*! \todo Bleeding eyes :-) */
+	tsig_rdata_set_fudge(tmp_tsig, KNOT_TSIG_FUDGE_DEFAULT);
 
 	/* Set original ID */
 	tsig_rdata_set_orig_id(tmp_tsig, knot_wire_get_id(msg));
@@ -572,27 +569,24 @@ int knot_tsig_sign(uint8_t *msg, size_t *msg_len,
 	if (ret != KNOT_EOK) {
 		dbg_tsig("TSIG: could not create wire or sign wire: %s\n",
 		         knot_strerror(ret));
-		knot_rrset_free(&tmp_tsig);
-		knot_rdata_free(&rdata);
-		
+		knot_rrset_deep_free(&tmp_tsig, 1, 1, 1);
 		return ret;
 	}
 
 	/* Set the digest. */
 	size_t tsig_wire_len = msg_max_len - *msg_len;
+	dbg_tsig("TSIG: msg_len=%zu, msg_max_len=%zu, tsig_max_len=%zu\n",
+		 *msg_len, msg_max_len, tsig_wire_len);
 	int rr_count = 0;
 	tsig_rdata_set_mac(tmp_tsig, digest_tmp_len, digest_tmp);
 
-	//knot_rrset_dump(tmp_tsig, 1);
-
 	/* Write RRSet to wire */
 	ret = knot_rrset_to_wire(tmp_tsig, msg + *msg_len,
 	                         &tsig_wire_len, &rr_count);
 	if (ret != KNOT_EOK) {
-		dbg_tsig_detail("TSIG: rrset_to_wire = %s\n", knot_strerror(ret));
+		dbg_tsig("TSIG: rrset_to_wire = %s\n", knot_strerror(ret));
 		*digest_len = 0;
-		knot_rrset_free(&tmp_tsig);
-		knot_rdata_free(&rdata);
+		knot_rrset_deep_free(&tmp_tsig, 1, 1, 1);
 		return ret;
 	}
 
@@ -617,7 +611,7 @@ int knot_tsig_sign_next(uint8_t *msg, size_t *msg_len, size_t msg_max_len,
                         size_t to_sign_len)
 {
 	if (!msg || !msg_len || !key || !key || !digest || !digest_len) {
-		return KNOT_EBADARG;
+		return KNOT_EINVAL;
 	}
 	
 	uint8_t digest_tmp[KNOT_TSIG_MAX_DIGEST_SIZE];
@@ -633,7 +627,7 @@ int knot_tsig_sign_next(uint8_t *msg, size_t *msg_len, size_t msg_max_len,
 	/* Create rdata for TSIG RR. */
 	knot_rdata_t *rdata = knot_rdata_new();
 	if (!rdata) {
-		dbg_tsig_detail("TSIG: rdata = NULL\n");
+		dbg_tsig("TSIG: rdata = NULL\n");
 		knot_rrset_free(&tmp_tsig);
 		return KNOT_ENOMEM;
 	}
@@ -642,7 +636,7 @@ int knot_tsig_sign_next(uint8_t *msg, size_t *msg_len, size_t msg_max_len,
 
 	ret = knot_rrset_add_rdata(tmp_tsig, rdata);
 	if (ret != KNOT_EOK) {
-		dbg_tsig_detail("TSIG: could not add rdata\n");
+		dbg_tsig("TSIG: could not add rdata\n");
 		knot_rrset_free(&tmp_tsig);
 		knot_rdata_free(&rdata);
 		return ret;
@@ -656,7 +650,7 @@ int knot_tsig_sign_next(uint8_t *msg, size_t *msg_len, size_t msg_max_len,
 	knot_rdata_item_t *items =
 		malloc(sizeof(knot_rdata_item_t) * desc->length);
 	if (!items) {
-		dbg_tsig_detail("TSIG: items = NULL\n");
+		dbg_tsig("TSIG: items = NULL\n");
 		ERR_ALLOC_FAILED;
 		knot_rrset_free(&tmp_tsig);
 		knot_rdata_free(&rdata);
@@ -667,7 +661,8 @@ int knot_tsig_sign_next(uint8_t *msg, size_t *msg_len, size_t msg_max_len,
 
 	ret = knot_rdata_set_items(rdata, items, desc->length);
 	if (ret != KNOT_EOK) {
-		dbg_tsig_detail("TSIG: rdata_set_items returned %s\n", knot_strerror(ret));
+		dbg_tsig("TSIG: rdata_set_items returned %s\n",
+		         knot_strerror(ret));
 		knot_rrset_free(&tmp_tsig);
 		knot_rdata_free(&rdata);
 		free(items);
@@ -676,7 +671,7 @@ int knot_tsig_sign_next(uint8_t *msg, size_t *msg_len, size_t msg_max_len,
 	free(items);
 
 	tsig_rdata_store_current_time(tmp_tsig);
-	tsig_rdata_set_fudge(tmp_tsig, 300);
+	tsig_rdata_set_fudge(tmp_tsig, KNOT_TSIG_FUDGE_DEFAULT);
 	
 	/* Create wire to be signed. */
 	size_t wire_len = prev_digest_len + to_sign_len
@@ -739,8 +734,8 @@ int knot_tsig_sign_next(uint8_t *msg, size_t *msg_len, size_t msg_max_len,
 	/* Set other data. */
 	tsig_rdata_set_other_data(tmp_tsig, 0, NULL);
 
-	dbg_tsig_detail("Message max length: %zu, message length: %zu\n",
-			msg_max_len, *msg_len);
+	dbg_tsig_verb("Message max length: %zu, message length: %zu\n",
+	              msg_max_len, *msg_len);
 
 	size_t tsig_wire_size = msg_max_len - *msg_len;
 	int rr_count = 0;
@@ -755,7 +750,7 @@ int knot_tsig_sign_next(uint8_t *msg, size_t *msg_len, size_t msg_max_len,
 	/* This should not happen, at least one rr has to be converted. */
 	if (rr_count == 0) {
 		knot_rrset_deep_free(&tmp_tsig, 1, 1, 1);
-		return KNOT_EBADARG;
+		return KNOT_EINVAL;
 	}
 
 	knot_rrset_deep_free(&tmp_tsig, 1, 1, 1);
@@ -779,7 +774,7 @@ static int knot_tsig_check_digest(const knot_rrset_t *tsig_rr,
                                   int use_times)
 {
 	if (!tsig_rr || !wire || !tsig_key) {
-		return KNOT_EBADARG;
+		return KNOT_EINVAL;
 	}
 
 	/* Check time signed. */
@@ -788,7 +783,7 @@ static int knot_tsig_check_digest(const knot_rrset_t *tsig_rr,
 		return ret;
 	}
 
-	dbg_tsig("TSIG: time checked.\n");
+	dbg_tsig_verb("TSIG: time checked.\n");
 
 	/* Check that libknot knows the algorithm. */
 	ret = knot_tsig_check_algorithm(tsig_rr);
@@ -796,7 +791,7 @@ static int knot_tsig_check_digest(const knot_rrset_t *tsig_rr,
 		return ret;
 	}
 
-	dbg_tsig("TSIG: algorithm checked.\n");
+	dbg_tsig_verb("TSIG: algorithm checked.\n");
 
 	/* Check that key is valid, ie. the same as given in args. */
 	ret = knot_tsig_check_key(tsig_rr, tsig_key);
@@ -804,22 +799,15 @@ static int knot_tsig_check_digest(const knot_rrset_t *tsig_rr,
 		return ret;
 	}
 
-	dbg_tsig("TSIG: key validity checked.\n");
+	dbg_tsig_verb("TSIG: key validity checked.\n");
 
 	/* Time OK algorithm OK, key name OK - do digest. */
 	/* Calculate the size of TSIG RR. */
 	size_t tsig_len = tsig_wire_actsize(tsig_rr);
 
-	dbg_tsig_detail("TSIG: check digest: wire before strip: \n");
-	//dbg_tsig_hex_detail(wire, size);
-
 	/* Strip the TSIG. */
 	size -= tsig_len;
 
-	dbg_tsig_detail("TSIG: check digest: wire after strip (stripped %zu):\n",
-	                tsig_len);
-	//dbg_tsig_hex_detail(wire, size);
-
 	uint8_t *wire_to_sign = malloc(sizeof(uint8_t) * size);
 	if (!wire_to_sign) {
 		ERR_ALLOC_FAILED;
@@ -858,16 +846,7 @@ static int knot_tsig_check_digest(const knot_rrset_t *tsig_rr,
 		return ret;
 	}
 
-//	uint8_t digest_tmp[KNOT_TSIG_MAX_DIGEST_SIZE];
-//	size_t digest_tmp_len = 0;
-//	ret = knot_tsig_compute_digest(wire, size, digest_tmp,
-//	                               &digest_tmp_len, tsig_key);
-//	if (ret != KNOT_EOK) {
-//		dbg_tsig("TSIG: digest could not be calculated\n");
-//		return ret;
-//	}
-
-	dbg_tsig("TSIG: digest calculated\n");
+	dbg_tsig_verb("TSIG: digest calculated\n");
 
 	/* Compare MAC from TSIG RR RDATA with just computed digest. */
 
@@ -880,17 +859,16 @@ static int knot_tsig_check_digest(const knot_rrset_t *tsig_rr,
 	const uint8_t *tsig_mac = tsig_rdata_mac(tsig_rr);
 
 	if (mac_length != tsig_alg_digest_length(alg)) {
-		dbg_tsig("TSIG: calculated digest length and given length do not match!\n");
+		dbg_tsig("TSIG: calculated digest length and given length do "
+		         "not match!\n");
 		return KNOT_TSIG_EBADSIG;
 	}
 
-//	assert(tsig_alg_digest_length(alg) == mac_length);
+	dbg_tsig_verb("TSIG: calc digest : ");
+	dbg_tsig_hex_verb(digest_tmp, digest_tmp_len);
 
-	dbg_tsig("TSIG: calc digest : ");
-	dbg_tsig_hex(digest_tmp, digest_tmp_len);
-
-	dbg_tsig("TSIG: given digest: ");
-	dbg_tsig_hex(tsig_mac, mac_length);
+	dbg_tsig_verb("TSIG: given digest: ");
+	dbg_tsig_hex_verb(tsig_mac, mac_length);
 
 	if (strncasecmp((char *)(tsig_mac), (char *)digest_tmp,
 	                mac_length) != 0) {
@@ -904,7 +882,7 @@ int knot_tsig_server_check(const knot_rrset_t *tsig_rr,
                            const uint8_t *wire, size_t size,
                            const knot_key_t *tsig_key)
 {
-	dbg_tsig_verb("tsig_server_check()\n");
+	dbg_tsig("tsig_server_check()\n");
 	return knot_tsig_check_digest(tsig_rr, wire, size, NULL, 0, tsig_key,
 	                              0, 0);
 }
@@ -915,7 +893,7 @@ int knot_tsig_client_check(const knot_rrset_t *tsig_rr,
                            const knot_key_t *tsig_key,
                            uint64_t prev_time_signed)
 {
-	dbg_tsig_verb("tsig_client_check()\n");
+	dbg_tsig("tsig_client_check()\n");
 	return knot_tsig_check_digest(tsig_rr, wire, size, request_mac,
 	                              request_mac_len, tsig_key,
 	                              prev_time_signed, 0);
@@ -928,9 +906,7 @@ int knot_tsig_client_check_next(const knot_rrset_t *tsig_rr,
                                 const knot_key_t *tsig_key,
                                 uint64_t prev_time_signed)
 {
-//	return knot_tsig_client_check(tsig_rr, wire, size, prev_digest,
-//	                              prev_digest_len, tsig_key);
-	dbg_tsig_verb("tsig_client_check_next()\n");
+	dbg_tsig("tsig_client_check_next()\n");
 	return knot_tsig_check_digest(tsig_rr, wire, size, prev_digest,
 	                              prev_digest_len, tsig_key,
 	                              prev_time_signed, 1);
@@ -943,30 +919,32 @@ int knot_tsig_add(uint8_t *msg, size_t *msg_len, size_t msg_max_len,
 	/*! \todo Revise!! */
 
 	if (!msg || !msg_len || !tsig_rr) {
-		return KNOT_EBADARG;
+		return KNOT_EINVAL;
 	}
 
 	/*! \todo What key to use, when we do not sign? Does this even work? */
 	knot_dname_t *key_name =
 			knot_dname_deep_copy(knot_rrset_owner(tsig_rr));
 	if (key_name == NULL) {
-		dbg_tsig_detail("TSIG: failed to copy owner\n");
+		dbg_tsig("TSIG: failed to copy owner\n");
 		return KNOT_ERROR;
 	}
 
 	knot_rrset_t *tmp_tsig =
-		knot_rrset_new(key_name,
-			       KNOT_RRTYPE_TSIG, KNOT_CLASS_ANY, 0);
+		knot_rrset_new(key_name, KNOT_RRTYPE_TSIG, KNOT_CLASS_ANY, 0);
 	if (!tmp_tsig) {
-		dbg_tsig_detail("TSIG: tmp_tsig = NULL\n");
+		dbg_tsig("TSIG: tmp_tsig = NULL\n");
 		knot_dname_free(&key_name);
 		return KNOT_ENOMEM;
 	}
+	
+	/* Already referenced in tmp_tsig, release. */
+	knot_dname_release(key_name);
 
 	/* Create rdata for TSIG RR. */
 	knot_rdata_t *rdata = knot_rdata_new();
 	if (!rdata) {
-		dbg_tsig_detail("TSIG: rdata = NULL\n");
+		dbg_tsig("TSIG: rdata = NULL\n");
 		knot_rrset_deep_free(&tmp_tsig, 1, 1, 1);
 		return KNOT_ENOMEM;
 	}
@@ -981,8 +959,7 @@ int knot_tsig_add(uint8_t *msg, size_t *msg_len, size_t msg_max_len,
 	knot_rdata_item_t *items =
 		malloc(sizeof(knot_rdata_item_t) * desc->length);
 	if (items == NULL) {
-		dbg_tsig_detail("TSIG: items = NULL\n");
-		ERR_ALLOC_FAILED;
+		dbg_tsig("TSIG: items = NULL\n");
 		knot_rrset_deep_free(&tmp_tsig, 1, 1, 1);
 		return KNOT_ENOMEM;
 	}
@@ -992,8 +969,8 @@ int knot_tsig_add(uint8_t *msg, size_t *msg_len, size_t msg_max_len,
 	int ret = knot_rdata_set_items(rdata, items, desc->length);
 	free(items);
 	if (ret != KNOT_EOK) {
-		dbg_tsig_detail("TSIG: rdata_set_items returned %s\n",
-		                knot_strerror(ret));
+		dbg_tsig("TSIG: rdata_set_items returned %s\n",
+		         knot_strerror(ret));
 		knot_rrset_deep_free(&tmp_tsig, 1, 1, 1);
 		return ret;
 	}
@@ -1001,14 +978,18 @@ int knot_tsig_add(uint8_t *msg, size_t *msg_len, size_t msg_max_len,
 	knot_dname_t *alg_name =
 			knot_dname_deep_copy(tsig_rdata_alg_name(tsig_rr));
 	if (alg_name == NULL) {
-		dbg_tsig_detail("TSIG: failed to copy alg name\n");
+		dbg_tsig("TSIG: failed to copy alg name\n");
 		knot_rrset_deep_free(&tmp_tsig, 1, 1, 1);
 		return KNOT_ERROR;
 	}
 
 	tsig_rdata_set_alg_name(tmp_tsig, alg_name);
 	tsig_rdata_set_time_signed(tmp_tsig, tsig_rdata_time_signed(tsig_rr));
-	tsig_rdata_set_fudge(tmp_tsig, tsig_rdata_fudge(tsig_rr));
+
+	/* Comparing to BIND it was found out that the Fudge should always be
+	 * set to the server's value.
+	 */
+	tsig_rdata_set_fudge(tmp_tsig, KNOT_TSIG_FUDGE_DEFAULT);
 	tsig_rdata_set_mac(tmp_tsig, 0, NULL);
 	knot_dname_release(alg_name); /* Already copied in tsig_rdata_set_alg_name() */
 
@@ -1029,13 +1010,13 @@ int knot_tsig_add(uint8_t *msg, size_t *msg_len, size_t msg_max_len,
 	ret = knot_rrset_to_wire(tmp_tsig, msg + *msg_len,
 	                         &tsig_wire_len, &rr_count);
 	if (ret != KNOT_EOK) {
-		dbg_tsig_detail("TSIG: rrset_to_wire = %s\n", knot_strerror(ret));
+		dbg_tsig("TSIG: rrset_to_wire = %s\n", knot_strerror(ret));
 		knot_rrset_deep_free(&tmp_tsig, 1, 1, 1);
 		return ret;
 	}
 
+	/* key_name already referenced in RRSet, no need to free separately. */
 	knot_rrset_deep_free(&tmp_tsig, 1, 1, 1);
-	knot_dname_release(key_name);
 
 	*msg_len += tsig_wire_len;
 
