@@ -78,7 +78,7 @@ enum tsig_consts {
 	                          + 6			// time signed
 };
 
-/*! TSIG errors are defined in util/error.h
+/*! TSIG errors are defined in common/errcode.h
  *  and present negative value of the TSIG error to
  *  comply with other parts of the library.
  *
@@ -140,6 +140,8 @@ uint16_t tsig_alg_digest_length(tsig_algorithm_t alg);
 size_t tsig_wire_maxsize(const knot_key_t *key);
 size_t tsig_wire_actsize(const knot_rrset_t *tsig);
 
+int tsig_rdata_is_ok(const knot_rrset_t *tsig);
+
 #endif /* _KNOT_TSIG_H_ */
 
 /*! @} */
