@@ -20,7 +20,6 @@
 #include "updates/changesets.h"
 #include "util/debug.h"
 #include "packet/packet.h"
-#include "util/error.h"
 #include "consts.h"
 
 /*----------------------------------------------------------------------------*/
@@ -97,7 +96,7 @@ static int knot_ddns_add_prereq_rrset(const knot_rrset_t *rrset,
 	}
 
 	knot_rrset_t *new_rrset = NULL;
-	ret = knot_rrset_deep_copy(rrset, &new_rrset);
+	ret = knot_rrset_deep_copy(rrset, &new_rrset, 0);
 	if (ret != KNOT_EOK) {
 		return ret;
 	}
@@ -202,7 +201,7 @@ static int knot_ddns_add_update(knot_changeset_t *changeset,
 	 *        copy.
 	 */
 	knot_rrset_t *rrset_copy;
-	ret = knot_rrset_deep_copy(rrset, &rrset_copy);
+	ret = knot_rrset_deep_copy(rrset, &rrset_copy, 0);
 	if (ret != KNOT_EOK) {
 		return ret;
 	}
@@ -410,7 +409,7 @@ int knot_ddns_check_zone(const knot_zone_t *zone, knot_packet_t *query,
                          uint8_t *rcode)
 {
 	if (zone == NULL || query == NULL || rcode == NULL) {
-		return KNOT_EBADARG;
+		return KNOT_EINVAL;
 	}
 
 	if (knot_packet_qtype(query) != KNOT_RRTYPE_SOA) {
@@ -448,7 +447,7 @@ int knot_ddns_process_prereqs(knot_packet_t *query,
 	 */
 
 	if (query == NULL || prereqs == NULL || rcode == NULL) {
-		return KNOT_EBADARG;
+		return KNOT_EINVAL;
 	}
 
 	// allocate space for the prerequisities
@@ -573,7 +572,7 @@ int knot_ddns_process_update(knot_packet_t *query,
 	// it will be distinguished when applying to the zone
 
 	if (query == NULL || changeset == NULL || rcode == NULL) {
-		return KNOT_EBADARG;
+		return KNOT_EINVAL;
 	}
 
 	*changeset = (knot_changeset_t *)calloc(1, sizeof(knot_changeset_t));
