@@ -21,7 +21,6 @@
 #include <stddef.h>
 
 #include "zone/dname-table.h"
-#include "util/error.h"
 
 /*!< Tree functions. */
 TREE_DEFINE(dname_table_node, avl);
@@ -167,7 +166,7 @@ int knot_dname_table_add_dname(knot_dname_table_t *table,
                                knot_dname_t *dname)
 {
 	if (dname == NULL || table == NULL) {
-		return KNOT_EBADARG;
+		return KNOT_EINVAL;
 	}
 
 	/* Node for insertion has to be created */
@@ -199,7 +198,7 @@ int knot_dname_table_add_dname_check(knot_dname_table_t *table,
 	knot_dname_t *found_dname = NULL;
 
 	if (table == NULL || dname == NULL || *dname == NULL) {
-		return KNOT_EBADARG;
+		return KNOT_EINVAL;
 	}
 
 	/* Fetch dname, need to release it later. */
