@@ -27,12 +27,17 @@
 #include "dname.h"
 #include "util/descriptor.h"
 #include "nsec3.h"
-#include "util/error.h"
 #include "util/debug.h"
 #include "util/utils.h"
 #include "hash/cuckoo-hash-table.h"
 #include "zone/zone-contents.h"
 
+/*! \brief Adaptor for knot_zone_deep_free() */
+static void knot_zone_dtor(struct ref_t *p) {
+	knot_zone_t *z = (knot_zone_t *)p;
+	knot_zone_deep_free(&z, 0);
+}
+
 /*----------------------------------------------------------------------------*/
 /* API functions                                                              */
 /*----------------------------------------------------------------------------*/
@@ -55,6 +60,13 @@ knot_zone_t *knot_zone_new_empty(knot_dname_t *name)
 	// save the zone name
 	dbg_zone("Setting zone name.\n");
 	zone->name = name;
+	
+	/* Initialize reference counting. */
+	ref_init(&zone->ref, knot_zone_dtor);
+	
+	/* Set reference counter to 1, caller should release it after use. */
+	knot_zone_retain(zone);
+	
 	return zone;
 }
 
@@ -112,7 +124,7 @@ const knot_zone_contents_t *knot_zone_contents(
 time_t knot_zone_version(const knot_zone_t *zone)
 {
 	if (zone == NULL) {
-		return KNOT_EBADARG;
+		return KNOT_EINVAL;
 	}
 
 	return zone->version;
@@ -133,14 +145,18 @@ void knot_zone_set_version(knot_zone_t *zone, time_t version)
 
 short knot_zone_is_master(const knot_zone_t *zone)
 {
-	return zone->master;
+	return zone->flags & KNOT_ZONE_MASTER;
 }
 
 /*----------------------------------------------------------------------------*/
 
 void knot_zone_set_master(knot_zone_t *zone, short master)
 {
-	zone->master = master;
+	if (master) {
+		zone->flags |= KNOT_ZONE_MASTER;
+	} else {
+		zone->flags &= ~KNOT_ZONE_MASTER;
+	}
 }
 
 /*----------------------------------------------------------------------------*/
@@ -175,6 +191,7 @@ knot_zone_contents_t *knot_zone_switch_contents(knot_zone_t *zone,
 
 	knot_zone_contents_t *old_contents =
 		rcu_xchg_pointer(&zone->contents, new_contents);
+
 	return old_contents;
 }
 
@@ -192,7 +209,7 @@ void knot_zone_free(knot_zone_t **zone)
 	    && !knot_zone_contents_gen_is_old((*zone)->contents)) {
 		// zone is in the middle of an update, report
 		dbg_zone("Destroying zone that is in the middle of an "
-		                  "update.\n");
+		         "update.\n");
 	}
 
 	knot_dname_release((*zone)->name);
@@ -221,7 +238,7 @@ void knot_zone_deep_free(knot_zone_t **zone, int destroy_dname_table)
 	    && !knot_zone_contents_gen_is_old((*zone)->contents)) {
 		// zone is in the middle of an update, report
 		dbg_zone("Destroying zone that is in the middle of an "
-		                  "update.\n");
+		         "update.\n");
 	}
 
 dbg_zone_exec(
@@ -241,3 +258,21 @@ dbg_zone_exec(
 	free(*zone);
 	*zone = NULL;
 }
+
+void knot_zone_set_dtor(knot_zone_t *zone, int (*dtor)(struct knot_zone *))
+{
+	if (zone != NULL) {
+		zone->dtor = dtor;
+	}
+}
+
+void knot_zone_set_flag(knot_zone_t *zone, knot_zone_flag_t flag, unsigned on)
+{
+	if (zone != NULL) {
+		if (on) {
+			zone->flags |= flag;
+		} else {
+			zone->flags &= ~flag;
+		}
+	}
+}
