@@ -20,7 +20,6 @@
 #include "libknot/common.h"
 #include "libknot/zone/dname-table.h"
 #include "libknot/zone/zone.h"
-#include "libknot/util/error.h"
 #include "libknot/zone/node.h"
 
 static int knot_zone_tests_count(int argc, char *argv[]);
@@ -95,13 +94,13 @@ static int test_zone_create(knot_zone_contents_t **zone)
 
 	if ((*zone) == NULL) {
 		diag("zone: Failed to create zone.");
-		knot_node_free(&node, 0);
+		knot_node_free(&node);
 		return 0;
 	}
 
 	if ((*zone)->apex != node) {
 		diag("zone: Zone apex not set right.");
-		knot_node_free(&node, 0);
+		knot_node_free(&node);
 		return 0;
 	}
 
@@ -133,7 +132,7 @@ static int test_zone_add_node(knot_zone_contents_t *zone, int nsec3)
 		                   : knot_zone_contents_add_node(zone, node, 0, 0, 0))) != 0) {
 			diag("zone: Failed to insert node into zone (returned"
 			     " %d).", res);
-			knot_node_free(&node, 0);
+			knot_node_free(&node);
 			++errors;
 		}
 		/* TODO check values in the node as well */
@@ -157,7 +156,7 @@ static int test_zone_add_node(knot_zone_contents_t *zone, int nsec3)
 			     KNOT_EBADZONE);
 			++errors;
 		}
-		knot_node_free(&node, 0);
+		knot_node_free(&node);
 	}
 
 	//note("NULL zone");
@@ -172,23 +171,23 @@ static int test_zone_add_node(knot_zone_contents_t *zone, int nsec3)
 	}
 
 	if ((res = ((nsec3) ? knot_zone_contents_add_nsec3_node(NULL, node, 0, 0, 0)
-		: knot_zone_contents_add_node(NULL, node, 0, 0, 0))) != KNOT_EBADARG) {
+		: knot_zone_contents_add_node(NULL, node, 0, 0, 0))) != KNOT_EINVAL) {
 		diag("zone: Inserting node to NULL zone did not result in"
 		     "proper return value (%d instead of %d)", res,
-		     KNOT_EBADARG);
+		     KNOT_EINVAL);
 		++errors;
 	}
 
-	knot_node_free(&node, 0);
+	knot_node_free(&node);
 
 	//note("NULL node");
 	note("Inserting NULL node...\n");
 
 	if ((res = ((nsec3) ? knot_zone_contents_add_nsec3_node(zone, NULL, 0, 0, 0)
-		: knot_zone_contents_add_node(zone, NULL, 0, 0, 0))) != KNOT_EBADARG) {
+		: knot_zone_contents_add_node(zone, NULL, 0, 0, 0))) != KNOT_EINVAL) {
 		diag("zone: Inserting NULL node to zone did not result in"
 		     "proper return value (%d instead of %d)", res,
-		     KNOT_EBADARG);
+		     KNOT_EINVAL);
 		++errors;
 	}
 
@@ -211,7 +210,7 @@ static int test_zone_add_node(knot_zone_contents_t *zone, int nsec3)
 			++errors;
 		}
 
-		knot_node_free(&node, 0);
+		knot_node_free(&node);
 	}
 
 	// check if all nodes are inserted
@@ -522,32 +521,32 @@ static int test_zone_shallow_copy()
 		knot_node_new(apex_dname, NULL, 0);
 	assert(apex_node);
 	lives_ok({
-		if (knot_zone_contents_shallow_copy(NULL, NULL) != KNOT_EBADARG) {
+		if (knot_zone_contents_shallow_copy(NULL, NULL) != KNOT_EINVAL) {
 			diag("Calling zone_shallow_copy with NULL "
-			     "arguments did not return KNOT_EBADARG!");
+			     "arguments did not return KNOT_EINVAL!");
 			errors++;
 		}
 		lived = 1;
 		lived = 0;
 		knot_zone_contents_t *zone = knot_zone_contents_new(apex_node,
 									0, 1, 0);
-		if (knot_zone_contents_shallow_copy(zone, NULL) != KNOT_EBADARG) {
+		if (knot_zone_contents_shallow_copy(zone, NULL) != KNOT_EINVAL) {
 			diag("Calling zone_shallow_copy with NULL destination "
-			     "zone argument did not return KNOT_EBADARG!");
+			     "zone argument did not return KNOT_EINVAL!");
 			errors++;
 		}
 		lived = 1;
 		lived = 0;
-		if (knot_zone_contents_shallow_copy(NULL, &zone) != KNOT_EBADARG) {
+		if (knot_zone_contents_shallow_copy(NULL, &zone) != KNOT_EINVAL) {
 			diag("Calling zone_shallow_copy with NULL source "
-			     "zone argument did not return KNOT_EBADARG!");
+			     "zone argument did not return KNOT_EINVAL!");
 			errors++;
 		}
 		lived = 1;
 		lived = 0;
-		if (knot_zone_contents_shallow_copy(zone, &zone) != KNOT_EBADARG) {
+		if (knot_zone_contents_shallow_copy(zone, &zone) != KNOT_EINVAL) {
 			diag("Calling zone_shallow_copy with identical source "
-			 "and destination zone did not return KNOT_EBADARG!");
+			 "and destination zone did not return KNOT_EINVAL!");
 			errors++;
 		}
 		lived = 1;
