@@ -20,7 +20,6 @@
 #include "tests/libknot/realdata/libknot_tests_loader_realdata.h"
 #include "libknot/common.h"
 #include "libknot/zone/zone.h"
-#include "libknot/util/error.h"
 #include "libknot/zone/node.h"
 
 static int knot_zone_tests_count(int argc, char *argv[]);
@@ -77,7 +76,7 @@ static int test_zone_create(list node_list)
 			errors++;
 		}
 		knot_node_free_rrsets(node, 1);
-		knot_node_free(&node, 0);
+		knot_node_free(&node);
 	}
 
 	return (errors == 0);
