@@ -131,18 +131,11 @@ struct zparser {
 	knot_rrset_t *current_rrset; /*!< Current RRSet. */
 	knot_rdata_item_t *temporary_items; /*!< Temporary rdata items. */
 
-	/*!
-	 * \brief list of RRSIGs that were not inside their nodes in zone file
-	 */
-	rrset_list_t *rrsig_orphans;
-	unsigned long rrsig_orphan_count; /*!< RRSIG orphan count */
-
 	knot_dname_t *root_domain; /*!< Root domain name. */
 	slab_cache_t *parser_slab; /*!< Slab for parser. */
 	rrset_list_t *node_rrsigs; /*!< List of RRSIGs in current node. */
 
 	int rdata_count; /*!< Count of parsed rdata. */
-	int origin_directive; /*!< Set to 1 if $ORIGIN directive is present. */
 };
 
 typedef struct zparser zparser_type;
